﻿/*

  PDF viewer, merger and splitter using:
    SumatraPDF.exe
    PDFtk.exe

  Source files:
    PdfView.prg
    PdfView.rc
    PdfView_C.prg
    PdfView_Keybd.ch
    PdfView_LangStr.ch
    Resource\ButtonArrow.bmp
    Resource\SumatraPDF.ico

  Compiler: HMG 3.4.4

  Link with SumatraPDF library (libSumatraPDF.a) or include SumatraPDF.prg

*/

#define PDFVIEW_VERSION "2017-09-11"

#include "Directry.ch"
#include "hbver.ch"
#include "hmg.ch"
#include "PdfView_Keybd.ch"
#include "PdfView_LangStr.ch"

MEMVAR _HMG_SYSDATA
MEMVAR _HMG_DEFAULTICONNAME

STATIC snHFocusMenu
STATIC snHMenuMain
STATIC slFileRefresh

STATIC saTab
STATIC saPanel
STATIC slMenuActive
STATIC scDirStart

//saved in .recent
STATIC saSession
STATIC saRecent

//saved in .ini
STATIC slPdfView_Max
STATIC snPdfView_R
STATIC snPdfView_C
STATIC snPdfView_W
STATIC snPdfView_H
STATIC snFiles_W
STATIC snRecent_W
STATIC snRecent_H
STATIC slRecentNames
STATIC snRecentAmount
STATIC snZoom
STATIC slMenuBar
STATIC slToolbar
STATIC slBookmarks
STATIC slOpenAtOnce
STATIC slSessionRest
STATIC slTabGoToFile
STATIC slEscExit
STATIC slPassShow
STATIC scSumatraDir
STATIC scPDFtkDir
STATIC scFileDir
STATIC scFileLast
STATIC scLang


FUNCTION Main()
  LOCAL lSessionOnInit := .F.
  LOCAL lOnGotFocus    := .F.

  SET FONT TO "MS Shell Dlg", 8
  SET DEFAULT ICON TO "SumatraPDF"

  SettingsRead()

  IF IsProgramRunning()
    QUIT
  ENDIF

  slFileRefresh := .T.

  DEFINE WINDOW PdfView;
    ROW    snPdfView_R;
    COL    snPdfView_C;
    WIDTH  snPdfView_W;
    HEIGHT snPdfView_H;
    TITLE  "PdfView";
    MAIN;
    ON INIT      ((lSessionOnInit := (SessionOpen(1) .or. SessionOpen(3))), FileListRefresh(scFileLast), (lSessionOnInit := .F.));
    ON GOTFOCUS  ((lOnGotFocus := .T.), If(slFileRefresh, (FileListRefresh(PdfView.Files.CellEx(PdfView.Files.VALUE, F_NAME)), TabCloseAllEmpty()), NIL ), (lOnGotFocus := .F.));
    ON SIZE      PdfViewResize(.T.);
    ON MAXIMIZE  PdfViewResize(.T.);
    ON RELEASE   (SettingsWrite(), DestroyMenu(snHMenuMain))

    DEFINE GRID Files
      ROW            0
      COL            -1
      WIDTH          snFiles_W
      HEADERS        {NIL, NIL, NIL, NIL, NIL}
      WIDTHS         {snFiles_W - 4, 0, 0, 0, 0}
      CELLNAVIGATION .F.
      ONDBLCLICK     FileOpen(NIL, (GetKeyState(VK_CONTROL) < 0), (GetKeyState(VK_SHIFT) < 0))
      ONCHANGE       If(slOpenAtOnce .and. (! lSessionOnInit) .and. (! lOnGotFocus) .and. (! ("D" $ PdfView.Files.CellEx(PdfView.Files.VALUE, F_ATTR))), FileOpen(), NIL)
      ONKEY          FileOnKey()
      ONHEADCLICK    { { || FileChooseDir()} }
    END GRID

    DEFINE TAB Tabs;
      ROW    0;
      COL    0;
      WIDTH  0;
      HEIGHT 20
    END TAB

    DEFINE STATUSBAR
      STATUSITEM ""
      STATUSITEM "" WIDTH 140
    END STATUSBAR
  END WINDOW

  PdfView.Files.PAINTDOUBLEBUFFER := .T.
  ListView_ChangeExtendedStyle(PdfView.Files.HANDLE, LVS_EX_INFOTIP)
  HMG_ChangeWindowStyle(PdfView.Tabs.HANDLE, 0x8000 /*TCS_FOCUSNEVER*/, NIL, .F., .F.)

  TabNew(PanelNew())

  IF slPdfView_Max
    PdfView.MAXIMIZE
  ENDIF

  SetOnKey(.T.)
  SetLangInterface()
  PdfViewResize(.T.)
  ChangeWindowMessageFilter(PdfView.HANDLE, 74 /*WM_COPYDATA*/, 1 /*MSGFLT_ALLOW*/)
  InstallEventHandler("MainEventHandler")
  EventCreate("TabEventHandler", PdfView.Tabs.HANDLE)

  DEFINE TIMER PdfTimer PARENT PdfView INTERVAL 100 ACTION StatusSetPage(.F.)

  PdfView.ACTIVATE

RETURN NIL


FUNCTION SetLangInterface(cNewLang)
  LOCAL aLang
  LOCAL cName
  LOCAL cAction
  LOCAL n

  IF ! (scLang == cNewLang)
    aLang := LangStr()

    IF ! Empty(cNewLang)
      scLang := cNewLang

      IF slMenuBar
        ReleaseMainMenu("PdfView")
      ELSE
        DestroyMenu(snHMenuMain)
      ENDIF
    ENDIF

    DEFINE MAINMENU OF PdfView
      DEFINE POPUP LangStr(LS_File)
        MENUITEM LangStr(LS_OpenCurTab) + e"\tEnter"                 NAME Open         ACTION FileOpen()
        MENUITEM LangStr(LS_OpenNewTab) + e"\tShift+Enter"           NAME OpenTab      ACTION FileOpen(NIL, .F., .T.)
        SEPARATOR
        MENUITEM LangStr(LS_OpenPage) + e"...\tCtrlt+Enter"          NAME OpenPage     ACTION FileOpen(NIL, .T., .F.)
        MENUITEM LangStr(LS_OpenPageTab) + e"...\tCtrlt+Shift+Enter" NAME OpenPageTab  ACTION FileOpen(NIL, .T., .T.)
        SEPARATOR
        MENUITEM LangStr(LS_OpenSession)                             NAME FileSession  ACTION SessionOpen(4)
        MENUITEM LangStr(LS_RecentFiles) + e"...\tCtrlt+Shift+R"     NAME FileRecent   ACTION RecentFiles()
        SEPARATOR
        MENUITEM LangStr(LS_ChooseDir) + e"...\tCtrlt+Shift+D"       NAME FileDir      ACTION FileChooseDir()
        SEPARATOR
        MENUITEM LangStr(LS_RefreshList) + e"\tF5"                   NAME FileRefresh  ACTION FileListRefresh(PdfView.Files.CellEx(PdfView.Files.VALUE, F_NAME))
        SEPARATOR
        MENUITEM LangStr(LS_Exit) + e"\tAlt+F4"                      NAME FileExit     ACTION ThisWindow.RELEASE
      END POPUP
      DEFINE POPUP LangStr(LS_Document)
        MENUITEM LangStr(LS_SaveAs) + e"...\tCtrl+S"                 NAME DocSaveAs    ACTION Sumatra_FileSaveAs(PanelName())
        MENUITEM LangStr(LS_Print) + e"...\tCtrl+P"                  NAME DocPrint     ACTION Sumatra_FilePrint(PanelName())
        MENUITEM LangStr(LS_Proper) + e"...\tCtrl+D"                 NAME DocProper    ACTION Sumatra_FileProperties(PanelName())
        MENUITEM LangStr(LS_Close) + e"\tCtrl+W"                     NAME DocClose     ACTION TabClose()
        SEPARATOR
        MENUITEM LangStr(LS_SplitPages) + e"\tCtrl+Shift+P"          NAME DocSplit     ACTION PdfSplit()
        SEPARATOR
        MENUITEM LangStr(LS_ChooseDoc) + e"\tAlt+0"                  NAME DocChoose    ACTION TabMenu(.F.)
        SEPARATOR
        MENUITEM LangStr(LS_GoToFile) + e"\tCtrl+Shift+F"            NAME DocGoToFile  ACTION FileGoTo()
      END POPUP
      DEFINE POPUP LangStr(LS_Page)
        MENUITEM LangStr(LS_GoTo) + e"...\tCtrl+G"                   NAME PageGoTo     ACTION Sumatra_PageGoTo(PanelName())
        SEPARATOR
        MENUITEM LangStr(LS_Prev) + e"\tCtrl+<-"                     NAME PagePrev     ACTION Sumatra_PageGoTo(PanelName(), -1)
        MENUITEM LangStr(LS_Next) + e"\tCtrl+->"                     NAME PageNext     ACTION Sumatra_PageGoTo(PanelName(),  1)
        MENUITEM LangStr(LS_First) + e"\tHome"                       NAME PageFirst    ACTION Sumatra_PageGoTo(PanelName(), -2)
        MENUITEM LangStr(LS_Last) + e"\tEnd"                         NAME PageLast     ACTION Sumatra_PageGoTo(PanelName(),  2)
      END POPUP
      DEFINE POPUP LangStr(LS_Find)
        MENUITEM LangStr(LS_Text) + e"...\tCtrl+F"                   NAME FindText     ACTION Sumatra_FindText(PanelName())
        SEPARATOR
        MENUITEM LangStr(LS_PrevOccur) + e"\tShift+F3"               NAME FindPrev     ACTION Sumatra_FindText(PanelName(), -1)
        MENUITEM LangStr(LS_NextOccur) + e"\tF3"                     NAME FindNext     ACTION Sumatra_FindText(PanelName(),  1)
      END POPUP
      DEFINE POPUP LangStr(LS_Zoom)
        MENUITEM LangStr(LS_SizeDn) + e"\tCtrl+Minus"                NAME ZoomSizeDn   ACTION Sumatra_Zoom(PanelName(), -1)
        MENUITEM LangStr(LS_SizeUp) + e"\tCtrl+Plus"                 NAME ZoomSizeUp   ACTION Sumatra_Zoom(PanelName(),  1)
        MENUITEM LangStr(LS_ZoomFactor) + e"...\tCtrl+Y"             NAME ZoomFactor   ACTION Sumatra_Zoom(PanelName())
        SEPARATOR
        MENUITEM LangStr(LS_FitPage) + e"\tCtrl+0"                   NAME ZoomFitPage  ACTION SetZoom(2)
        MENUITEM LangStr(LS_ActualSize) + e"\tCtrl+1"                NAME ZoomActual   ACTION SetZoom(3)
        MENUITEM LangStr(LS_FitWidth) + e"\tCtrl+2"                  NAME ZoomFitWidth ACTION SetZoom(4)
      END POPUP
      DEFINE POPUP LangStr(LS_Rotate)
        MENUITEM LangStr(LS_Left) + e"\tCtrl+Shift+Minus"            NAME RotateLeft   ACTION Sumatra_Rotate(PanelName(), -1)
        MENUITEM LangStr(LS_Right) + e"\tCtrl+Shift+Plus"            NAME RotateRight  ACTION Sumatra_Rotate(PanelName(),  1)
        MENUITEM e"&180°\tCtrl+Shift+Num*"                           NAME Rotate180    ACTION Sumatra_Rotate(PanelName())
      END POPUP
      DEFINE POPUP LangStr(LS_View)
        MENUITEM LangStr(LS_MenuBar) + e"\tF9"                       NAME MenuBar      ACTION SetMenu(PdfView.HANDLE, If((slMenuBar := ! slMenuBar), snHMenuMain, 0))
        MENUITEM LangStr(LS_Toolbar) + e"\tF8"                       NAME Toolbar      ACTION Sumatra_Toolbar(PanelName(), ! Sumatra_Toolbar(PanelName()))
        DEFINE POPUP LangStr(LS_Bookmarks) NAME Bookmarks
          MENUITEM LangStr(LS_Show) + e"\tF12"                       NAME BookShow     ACTION Sumatra_Bookmarks(PanelName(), ! Sumatra_Bookmarks(PanelName()))
          SEPARATOR
          MENUITEM LangStr(LS_ExpandAll) + e"\tCtrl+F12"             NAME BookExpand   ACTION Sumatra_BookmarksExpand(PanelName(), .T.)
          MENUITEM LangStr(LS_CollapseAll) + e"\tAlt+F12"            NAME BookCollapse ACTION Sumatra_BookmarksExpand(PanelName(), .F.)
        END POPUP
      END POPUP
      DEFINE POPUP LangStr(LS_Settings)
        MENUITEM LangStr(LS_OpenAtOnce)                              NAME OpenAtOnce   ACTION (slOpenAtOnce  := ! slOpenAtOnce)
        MENUITEM LangStr(LS_RestSession)                             NAME RestSession  ACTION (slSessionRest := ! slSessionRest)
        MENUITEM LangStr(LS_TabGoToFile)                             NAME TabGoToFile  ACTION (slTabGoToFile := ! slTabGoToFile)
        MENUITEM LangStr(LS_EscExit)                                 NAME EscExit      ACTION ((slEscExit    := ! slEscExit), SetOnKey(.T.))
        SEPARATOR
        MENUITEM LangStr(LS_SumatraDir)                              NAME SumatraDir   ACTION InputToolDir(1)
        MENUITEM LangStr(LS_PDFtkDir)                                NAME PDFtkDir     ACTION InputToolDir(2)
        SEPARATOR
        DEFINE POPUP LangStr(LS_Language)
          FOR n := 1 TO Len(aLang)
            cName   := "Lang_" + aLang[n][3]
            cAction := "SetLangInterface('" + aLang[n][3] + "')"
            MENUITEM aLang[n][1] + Chr(9) + aLang[n][2]              NAME &cName       ACTION &cAction
            IF aLang[n][3] == scLang
              SetProperty("PdfView", cName, "CHECKED", .T.)
            ENDIF
          NEXT
        END POPUP
        SEPARATOR
        MENUITEM LangStr(LS_AboutPdfView) + e"\tF1"                  NAME AboutPdfView ACTION AboutPdfView()
        MENUITEM LangStr(LS_AboutSumatra) + e"\tShift+F1"            NAME AboutSumatra ACTION Sumatra_About(PanelName())
      END POPUP
    END MENU

    snHMenuMain := GetMenu(PdfView.HANDLE)

    IF ! slMenuBar
      SetMenu(PdfView.HANDLE, 0)
    ENDIF

    IF ! Empty(cNewLang)
      PdfView.REDRAW
      SessionReopen()
    ENDIF
  ENDIF

RETURN NIL


FUNCTION MenuCommandsEnable()
  LOCAL lPdfSelected := (PdfView.Files.VALUE > 0) .and. (! ("D" $ PdfView.Files.CellEx(PdfView.Files.VALUE, F_ATTR)))
  LOCAL cPanel       := PanelName()
  LOCAL nPages       := Sumatra_PageCount(cPanel)
  LOCAL lPdfOpened   := (Sumatra_FrameHandle(cPanel) != 0)
  LOCAL lPdfLoaded   := (nPages > 0)

  GetSumatraSettings()

  PdfView.Open.ENABLED         := lPdfSelected
  PdfView.OpenTab.ENABLED      := lPdfSelected
  PdfView.OpenPage.ENABLED     := lPdfSelected
  PdfView.OpenPageTab.ENABLED  := lPdfSelected
  PdfView.FileSession.ENABLED  := ! Empty(saSession)
  PdfView.DocSaveAs.ENABLED    := lPdfLoaded
  PdfView.DocPrint.ENABLED     := lPdfLoaded
  PdfView.DocProper.ENABLED    := lPdfLoaded
  PdfView.DocClose.ENABLED     := lPdfOpened
  PdfView.DocSplit.ENABLED     := (nPages > 1)
  PdfView.DocChoose.ENABLED    := (Len(saTab) > 1)
  PdfView.DocGoToFile.ENABLED  := lPdfOpened
  PdfView.PageGoto.ENABLED     := lPdfLoaded
  PdfView.PagePrev.ENABLED     := lPdfLoaded
  PdfView.PageNext.ENABLED     := lPdfLoaded
  PdfView.PageFirst.ENABLED    := lPdfLoaded
  PdfView.PageLast.ENABLED     := lPdfLoaded
  PdfView.FindText.ENABLED     := lPdfLoaded
  PdfView.FindPrev.ENABLED     := lPdfLoaded
  PdfView.FindNext.ENABLED     := lPdfLoaded
  PdfView.ZoomSizeDn.ENABLED   := lPdfLoaded
  PdfView.ZoomSizeUp.ENABLED   := lPdfLoaded
  PdfView.ZoomFactor.ENABLED   := lPdfLoaded
  PdfView.ZoomFitPage.ENABLED  := lPdfLoaded
  PdfView.ZoomActual.ENABLED   := lPdfLoaded
  PdfView.ZoomFitWidth.ENABLED := lPdfLoaded
  PdfView.Rotate180.ENABLED    := lPdfLoaded
  PdfView.RotateLeft.ENABLED   := lPdfLoaded
  PdfView.RotateRight.ENABLED  := lPdfLoaded
  PdfView.Toolbar.ENABLED      := lPdfOpened
  PdfView.Bookmarks.ENABLED    := Sumatra_BookmarksExist(cPanel)
  PdfView.BookExpand.ENABLED   := slBookmarks
  PdfView.BookCollapse.ENABLED := slBookmarks
  PdfView.AboutSumatra.ENABLED := lPdfOpened

  PdfView.ZoomFitPage.CHECKED  := (snZoom == 2)
  PdfView.ZoomActual.CHECKED   := (snZoom == 3)
  PdfView.ZoomFitWidth.CHECKED := ((snZoom != 2) .and. (snZoom != 3))
  PdfView.MenuBar.CHECKED      := slMenuBar
  PdfView.Toolbar.CHECKED      := slToolbar
  PdfView.Bookmarks.CHECKED    := slBookmarks
  PdfView.BookShow.CHECKED     := slBookmarks
  PdfView.OpenAtOnce.CHECKED   := slOpenAtOnce
  PdfView.RestSession.CHECKED  := slSessionRest
  PdfView.TabGoToFile.CHECKED  := slTabGoToFile
  PdfView.EscExit.CHECKED      := slEscExit

RETURN NIL


FUNCTION SetOnKey(lSet)

  IF lSet
    ON KEY TAB                    OF PdfView ACTION SetFocusNextCtl(.F.)
    ON KEY SHIFT+TAB              OF PdfView ACTION SetFocusNextCtl(.T.)
    ON KEY CONTROL+TAB            OF PdfView ACTION TabChange(-1)
    ON KEY CONTROL+SHIFT+TAB      OF PdfView ACTION TabChange(-2)
    ON KEY CONTROL+W              OF PdfView ACTION TabClose()
    ON KEY CONTROL+S              OF PdfView ACTION Sumatra_FileSaveAs(PanelName())
    ON KEY CONTROL+P              OF PdfView ACTION Sumatra_FilePrint(PanelName())
    ON KEY CONTROL+D              OF PdfView ACTION Sumatra_FileProperties(PanelName())
    ON KEY CONTROL+SHIFT+D        OF PdfView ACTION FileChooseDir()
    ON KEY CONTROL+SHIFT+F        OF PdfView ACTION FileGoTo()
    ON KEY CONTROL+SHIFT+P        OF PdfView ACTION PdfSplit()
    ON KEY CONTROL+SHIFT+R        OF PdfView ACTION RecentFiles()
    ON KEY F5                     OF PdfView ACTION FileListRefresh(PdfView.Files.CellEx(PdfView.Files.VALUE, F_NAME))
    ON KEY CONTROL+G              OF PdfView ACTION Sumatra_PageGoTo(PanelName())
    ON KEY CONTROL+F              OF PdfView ACTION Sumatra_FindText(PanelName())
    ON KEY SHIFT+F3               OF PdfView ACTION Sumatra_FindText(PanelName(), -1)
    ON KEY F3                     OF PdfView ACTION Sumatra_FindText(PanelName(),  1)
    ON KEY CONTROL+MINUS          OF PdfView ACTION Sumatra_Zoom(PanelName(), -1)
    ON KEY CONTROL+SUBTRACT       OF PdfView ACTION Sumatra_Zoom(PanelName(), -1)
    ON KEY CONTROL+PLUS           OF PdfView ACTION Sumatra_Zoom(PanelName(),  1)
    ON KEY CONTROL+ADD            OF PdfView ACTION Sumatra_Zoom(PanelName(),  1)
    ON KEY CONTROL+Y              OF PdfView ACTION Sumatra_Zoom(PanelName())
    ON KEY CONTROL+0              OF PdfView ACTION SetZoom(2)
    ON KEY CONTROL+NUMPAD0        OF PdfView ACTION SetZoom(2)
    ON KEY CONTROL+1              OF PdfView ACTION SetZoom(3)
    ON KEY CONTROL+NUMPAD1        OF PdfView ACTION SetZoom(3)
    ON KEY CONTROL+2              OF PdfView ACTION SetZoom(4)
    ON KEY CONTROL+NUMPAD2        OF PdfView ACTION SetZoom(4)
    ON KEY CONTROL+7              OF PdfView ACTION NIL
    ON KEY CONTROL+NUMPAD7        OF PdfView ACTION NIL
    ON KEY CONTROL+8              OF PdfView ACTION NIL
    ON KEY CONTROL+NUMPAD8        OF PdfView ACTION NIL
    ON KEY CONTROL+SHIFT+MINUS    OF PdfView ACTION Sumatra_Rotate(PanelName(), -1)
    ON KEY CONTROL+SHIFT+SUBTRACT OF PdfView ACTION Sumatra_Rotate(PanelName(), -1)
    ON KEY CONTROL+SHIFT+PLUS     OF PdfView ACTION Sumatra_Rotate(PanelName(),  1)
    ON KEY CONTROL+SHIFT+ADD      OF PdfView ACTION Sumatra_Rotate(PanelName(),  1)
    ON KEY CONTROL+SHIFT+MULTIPLY OF PdfView ACTION Sumatra_Rotate(PanelName())
    ON KEY CONTROL+SHIFT+DIVIDE   OF PdfView ACTION Sumatra_Rotate(PanelName())
    ON KEY F9                     OF PdfView ACTION SetMenu(PdfView.HANDLE, If((slMenuBar := ! slMenuBar), snHMenuMain, 0))
    ON KEY F8                     OF PdfView ACTION Sumatra_Toolbar(PanelName(), ! Sumatra_Toolbar(PanelName()))
    ON KEY CONTROL+F12            OF PdfView ACTION Sumatra_BookmarksExpand(PanelName(), .T.)
    ON KEY ALT+F12                OF PdfView ACTION Sumatra_BookmarksExpand(PanelName(), .F.)
    ON KEY ALT+RETURN             OF PdfView ACTION If(IsMaximized(PdfView.HANDLE), PdfView.RESTORE, PdfView.MAXIMIZE)
    ON KEY ALT+1                  OF PdfView ACTION TabChange(1)
    ON KEY ALT+2                  OF PdfView ACTION TabChange(2)
    ON KEY ALT+3                  OF PdfView ACTION TabChange(3)
    ON KEY ALT+4                  OF PdfView ACTION TabChange(4)
    ON KEY ALT+5                  OF PdfView ACTION TabChange(5)
    ON KEY ALT+6                  OF PdfView ACTION TabChange(6)
    ON KEY ALT+7                  OF PdfView ACTION TabChange(7)
    ON KEY ALT+8                  OF PdfView ACTION TabChange(8)
    ON KEY ALT+9                  OF PdfView ACTION TabChange(0)
    ON KEY ALT+0                  OF PdfView ACTION TabMenu(.F.)
    ON KEY ALT+NUMPAD1            OF PdfView ACTION TabChange(1)
    ON KEY ALT+NUMPAD2            OF PdfView ACTION TabChange(2)
    ON KEY ALT+NUMPAD3            OF PdfView ACTION TabChange(3)
    ON KEY ALT+NUMPAD4            OF PdfView ACTION TabChange(4)
    ON KEY ALT+NUMPAD5            OF PdfView ACTION TabChange(5)
    ON KEY ALT+NUMPAD6            OF PdfView ACTION TabChange(6)
    ON KEY ALT+NUMPAD7            OF PdfView ACTION TabChange(7)
    ON KEY ALT+NUMPAD8            OF PdfView ACTION TabChange(8)
    ON KEY ALT+NUMPAD9            OF PdfView ACTION TabChange(0)
    ON KEY ALT+NUMPAD0            OF PdfView ACTION TabMenu(.F.)
    ON KEY F1                     OF PdfView ACTION AboutPdfView()
    ON KEY SHIFT+F1               OF PdfView ACTION Sumatra_About(PanelName())

    IF slEscExit
      ON KEY ESCAPE      OF PdfView ACTION PdfView.RELEASE
    ELSE
      RELEASE KEY ESCAPE OF PdfView
    ENDIF
  ELSE
    RELEASE KEY TAB                    OF PdfView
    RELEASE KEY SHIFT+TAB              OF PdfView
    RELEASE KEY CONTROL+TAB            OF PdfView
    RELEASE KEY CONTROL+SHIFT+TAB      OF PdfView
    RELEASE KEY CONTROL+W              OF PdfView
    RELEASE KEY CONTROL+S              OF PdfView
    RELEASE KEY CONTROL+P              OF PdfView
    RELEASE KEY CONTROL+D              OF PdfView
    RELEASE KEY CONTROL+SHIFT+D        OF PdfView
    RELEASE KEY CONTROL+SHIFT+F        OF PdfView
    RELEASE KEY CONTROL+SHIFT+P        OF PdfView
    RELEASE KEY CONTROL+SHIFT+R        OF PdfView
    RELEASE KEY F5                     OF PdfView
    RELEASE KEY CONTROL+G              OF PdfView
    RELEASE KEY CONTROL+F              OF PdfView
    RELEASE KEY SHIFT+F3               OF PdfView
    RELEASE KEY F3                     OF PdfView
    RELEASE KEY CONTROL+MINUS          OF PdfView
    RELEASE KEY CONTROL+SUBTRACT       OF PdfView
    RELEASE KEY CONTROL+PLUS           OF PdfView
    RELEASE KEY CONTROL+ADD            OF PdfView
    RELEASE KEY CONTROL+Y              OF PdfView
    RELEASE KEY CONTROL+0              OF PdfView
    RELEASE KEY CONTROL+NUMPAD0        OF PdfView
    RELEASE KEY CONTROL+1              OF PdfView
    RELEASE KEY CONTROL+NUMPAD1        OF PdfView
    RELEASE KEY CONTROL+2              OF PdfView
    RELEASE KEY CONTROL+NUMPAD2        OF PdfView
    RELEASE KEY CONTROL+7              OF PdfView
    RELEASE KEY CONTROL+NUMPAD7        OF PdfView
    RELEASE KEY CONTROL+8              OF PdfView
    RELEASE KEY CONTROL+NUMPAD8        OF PdfView
    RELEASE KEY CONTROL+SHIFT+MINUS    OF PdfView
    RELEASE KEY CONTROL+SHIFT+SUBTRACT OF PdfView
    RELEASE KEY CONTROL+SHIFT+PLUS     OF PdfView
    RELEASE KEY CONTROL+SHIFT+ADD      OF PdfView
    RELEASE KEY CONTROL+SHIFT+MULTIPLY OF PdfView
    RELEASE KEY CONTROL+SHIFT+DIVIDE   OF PdfView
    RELEASE KEY F9                     OF PdfView
    RELEASE KEY F8                     OF PdfView
    RELEASE KEY CONTROL+F12            OF PdfView
    RELEASE KEY ALT+F12                OF PdfView
    RELEASE KEY ALT+RETURN             OF PdfView
    RELEASE KEY ALT+1                  OF PdfView
    RELEASE KEY ALT+2                  OF PdfView
    RELEASE KEY ALT+3                  OF PdfView
    RELEASE KEY ALT+4                  OF PdfView
    RELEASE KEY ALT+5                  OF PdfView
    RELEASE KEY ALT+6                  OF PdfView
    RELEASE KEY ALT+7                  OF PdfView
    RELEASE KEY ALT+8                  OF PdfView
    RELEASE KEY ALT+9                  OF PdfView
    RELEASE KEY ALT+0                  OF PdfView
    RELEASE KEY ALT+NUMPAD1            OF PdfView
    RELEASE KEY ALT+NUMPAD2            OF PdfView
    RELEASE KEY ALT+NUMPAD3            OF PdfView
    RELEASE KEY ALT+NUMPAD4            OF PdfView
    RELEASE KEY ALT+NUMPAD5            OF PdfView
    RELEASE KEY ALT+NUMPAD6            OF PdfView
    RELEASE KEY ALT+NUMPAD7            OF PdfView
    RELEASE KEY ALT+NUMPAD8            OF PdfView
    RELEASE KEY ALT+NUMPAD9            OF PdfView
    RELEASE KEY ALT+NUMPAD0            OF PdfView
    RELEASE KEY F1                     OF PdfView
    RELEASE KEY SHIFT+F1               OF PdfView
    RELEASE KEY ESCAPE                 OF PdfView
  ENDIF

RETURN NIL


FUNCTION FileOnKey()

  IF ! slMenuActive
    SWITCH HMG_GetLastVirtualKeyDown()
      CASE VK_F12
        HMG_CleanLastVirtualKeyDown()

        IF (GetKeyState(VK_CONTROL) >= 0) .and. (GetKeyState(VK_SHIFT) >= 0) .and. (GetKeyState(VK_MENU) >= 0)
          Sumatra_Bookmarks(PanelName(), ! Sumatra_Bookmarks(PanelName()))
          PdfView.Files.SETFOCUS
        ENDIF
        EXIT
      CASE VK_LEFT
        HMG_CleanLastVirtualKeyDown()

        IF (GetKeyState(VK_CONTROL) >= 0) .and. (GetKeyState(VK_SHIFT) >= 0) .and. (GetKeyState(VK_MENU) >= 0) .and. (PdfView.Files.CellEx(1, F_NAME) == "<..>")
          PdfView.Files.VALUE := 1
          FileOpen()
        ENDIF
        EXIT
      CASE VK_RIGHT
        HMG_CleanLastVirtualKeyDown()

        IF (GetKeyState(VK_CONTROL) >= 0) .and. (GetKeyState(VK_SHIFT) >= 0) .and. (GetKeyState(VK_MENU) >= 0) .and. ("D" $ PdfView.Files.CellEx(PdfView.Files.VALUE, F_ATTR)) .and. (! (PdfView.Files.CellEx(PdfView.Files.VALUE, F_NAME) == "<..>"))
          FileOpen()
        ENDIF
        EXIT
    ENDSWITCH
  ENDIF

RETURN NIL


FUNCTION FileMenu(nRow, nCol)
  LOCAL aRect
  LOCAL nHMenu
  LOCAL nCmd

  //menu from keyboard
  IF nRow == 0xFFFF
    SendMessage(PdfView.Files.HANDLE, 0x1013 /*LVM_ENSUREVISIBLE*/, PdfView.Files.VALUE - 1, 0)
  ENDIF

  aRect    := ListView_GetItemRect(PdfView.Files.HANDLE, PdfView.Files.VALUE - 1)
  aRect[4] := ClientToScreenRow(PdfView.Files.HANDLE, aRect[1] + aRect[4])
  aRect[1] := ClientToScreenRow(PdfView.Files.HANDLE, aRect[1])
  aRect[2] := ClientToScreenCol(PdfView.Files.HANDLE, aRect[2])

  IF nRow == 0xFFFF
    nRow := aRect[4]
    nCol := aRect[2]
  ELSEIF (nRow < aRect[1]) .or. (nRow > aRect[4])
    RETURN NIL
  ENDIF

  nHMenu := CreatePopupMenu()

  IF ! ("D" $ PdfView.Files.CellEx(PdfView.Files.VALUE, F_ATTR))
    AppendMenuString(nHMenu, 1, LangStr(LS_OpenCurTab) + e"\tEnter")
    AppendMenuString(nHMenu, 2, LangStr(LS_OpenNewTab) + e"\tShift+Enter")
    AppendMenuSeparator(nHMenu)
    AppendMenuString(nHMenu, 3, LangStr(LS_OpenPage)    + e"...\tCtrlt+Enter")
    AppendMenuString(nHMenu, 4, LangStr(LS_OpenPageTab) + e"...\tCtrlt+Shift+Enter")
    AppendMenuSeparator(nHMenu)
  ELSEIF ! (PdfView.Files.CellEx(PdfView.Files.VALUE, F_NAME) == "<..>")
    AppendMenuString(nHMenu, 5, LangStr(LS_GoToSubDir) + e"\t-> (Enter)")
  ENDIF

  IF ! ("K" $ PdfView.Files.CellEx(PdfView.Files.VALUE, F_ATTR))
    AppendMenuString(nHMenu, 6, LangStr(LS_GoToParentDir) + e"\t<-")
  ENDIF

  AppendMenuString(nHMenu, 7, LangStr(LS_ChooseDir) + e"...\tCtrlt+Shift+D")
  AppendMenuSeparator(nHMenu)
  AppendMenuString(nHMenu, 8, LangStr(LS_RefreshList) + e"\tF5")

  slMenuActive := .T.
  SetOnKey(.F.)

  nCmd := TrackPopupMenu2(nHMenu, 0x0180 /*TPM_NONOTIFY|TPM_RETURNCMD*/, nRow, nCol, PdfView.HANDLE)

  DestroyMenu(nHMenu)
  SetOnKey(.T.)
  slMenuActive := .F.

  SWITCH nCmd
    CASE 6
      PdfView.Files.VALUE := 1
    CASE 1
    CASE 5
      FileOpen()
      EXIT
    CASE 2
      FileOpen(NIL, .F., .T.)
      EXIT
    CASE 3
      FileOpen(NIL, .T., .F.)
      EXIT
    CASE 4
      FileOpen(NIL, .T., .T.)
      EXIT
    CASE 7
      FileChooseDir()
      EXIT
    CASE 8
      FileListRefresh(PdfView.Files.CellEx(PdfView.Files.VALUE, F_NAME))
      EXIT
  ENDSWITCH

RETURN NIL


FUNCTION TabEventHandler()
  LOCAL nHWnd   := EventHWND()
  LOCAL nMsg    := EventMSG()
  LOCAL nWParam := EventWPARAM()
  LOCAL nLParam := EventLPARAM()

  IF nMsg == 517 /*WM_RBUTTONUP*/
    TabChange(Tab_HitTest(nHWnd, nLParam))
    TabMenu(.T.)
    RETURN 0
  ELSEIF nMsg == 519 /*WM_MBUTTONDOWN*/
    IF nWParam == 0x10 /*MK_MBUTTON*/
      TabClose(Tab_HitTest(nHWnd, nLParam))
    ENDIF
    RETURN 0
  ENDIF

RETURN NIL


FUNCTION TabMenu(lMouse)
  LOCAL nHMenu
  LOCAL nRow, nCol
  LOCAL nCmd
  LOCAL n

  IF Len(saTab) > 1
    nHMenu := CreatePopupMenu()

    FOR n := 1 TO Len(saTab)
      AppendMenuString(nHMenu, n, Sumatra_FileName(PanelName(saTab[n])) + If(n < 9, e"\tAlt+" + HB_NtoS(n), If(n == Len(saTab), e"\tAlt+9", "")))
    NEXT

    xCheckMenuItem(nHMenu, PdfView.Tabs.VALUE)

    IF lMouse
      HMG_GetCursorPos(NIL, @nRow, @nCol)
    ELSE
      nRow := GetWindowRow(PdfView.Tabs.HANDLE) + GetWindowHeight(PdfView.Tabs.HANDLE)
      nCol := GetWindowCol(PdfView.Tabs.HANDLE)
    ENDIF

    slMenuActive := .T.
    SetOnKey(.F.)

    nCmd := TrackPopupMenu2(nHMenu, 0x0180 /*TPM_NONOTIFY|TPM_RETURNCMD*/, nRow, nCol, PdfView.HANDLE)

    DestroyMenu(nHMenu)
    SetOnKey(.T.)
    slMenuActive := .F.

    IF nCmd > 0
      TabChange(nCmd)
    ENDIF
  ENDIF

RETURN NIL


FUNCTION SetFocusNextCtl(lPrevious)
  LOCAL nHFocus := GetFocus()
  LOCAL cPanel  := PanelName()
  LOCAL nHFrame := Sumatra_FrameHandle(cPanel)
  LOCAL nHBook

  IF nHFrame != 0
    nHBook := Sumatra_BookmarksHandle(cPanel)

    IF nHFocus == PdfView.Files.HANDLE
      SetFocus(If(lPrevious, nHFrame, If(Sumatra_Bookmarks(cPanel), nHBook, nHFrame)))
    ELSEIF nHFocus == nHBook
      SetFocus(If(lPrevious, PdfView.Files.HANDLE, nHFrame))
    ELSE
      SetFocus(If(lPrevious, If(Sumatra_Bookmarks(cPanel), nHBook, PdfView.Files.HANDLE), PdfView.Files.HANDLE))
    ENDIF
  ENDIF

RETURN NIL


FUNCTION PdfViewResize(lAll)
  LOCAL cPanel  := PanelName()
  LOCAL nMainCW := PdfView.CLIENTAREAWIDTH
  LOCAL nMainCH := PdfView.CLIENTAREAHEIGHT - GetWindowHeight(PdfView.STATUSBAR.HANDLE)
  LOCAL nFilesW := PdfView.Files.WIDTH
  LOCAL nTabsH  := If(PdfView.Tabs.VISIBLE, PdfView.Tabs.HEIGHT, 0)
  LOCAL nFrameW := (GetProperty(cPanel, "WIDTH") - GetProperty(cPanel, "CLIENTAREAWIDTH")) / 2

  IF lAll
    IF (nMainCW - nFilesW) < (200 - nFrameW)
      nFilesW := nMainCW - 200 - nFrameW
      PdfView.Files.WIDTH := nFilesW
    ENDIF

    PdfView.Files.HEIGHT := nMainCH
    PdfView.Files.ColumnWIDTH(1) := nFilesW - 4 - If(PdfView.Files.ITEMCOUNT > ListViewGetCountPerPage(PdfView.Files.HANDLE), GetVScrollBarWidth(), 0)

    PdfView.Tabs.COL   := nFilesW
    PdfView.Tabs.WIDTH := nMainCW - nFilesW

    SendMessage(PdfView.Files.HANDLE, 0x1013 /*LVM_ENSUREVISIBLE*/, PdfView.Files.VALUE - 1, 0)
  ENDIF

  //bug in MHG: SetProperty(cPanel, "ROW", nRow), SetProperty(cPanel, "COL", nCol)
  //reported: http://www.hmgforum.com/viewtopic.php?f=20&t=5178
  SetProperty(cPanel, "ROW", ClientToScreenRow(GetParent(GetFormHandle(cPanel)), nTabsH))
  SetProperty(cPanel, "COL", ClientToScreenCol(GetParent(GetFormHandle(cPanel)), nFilesW - 1))
  SetProperty(cPanel, "WIDTH", nMainCW - (nFilesW - 1) + nFrameW)
  SetProperty(cPanel, "HEIGHT", nMainCH - nTabsH)

  Sumatra_FrameAdjust(cPanel)

RETURN NIL


FUNCTION PanelNew()
  LOCAL nPanel
  LOCAL cPanel

  IF (Len(saTab) == 1) .and. saPanel[saTab[1]]
    nPanel := saTab[1]
  ELSE
    nPanel := aScan(saPanel, .T.)
  ENDIF

  IF nPanel == 0
    aAdd(saPanel, .T.)

    nPanel := Len(saPanel)
    cPanel := PanelName(nPanel)

    DEFINE WINDOW &cPanel;
      PARENT   PdfView;
      ROW      0xFFFF;
      COL      0xFFFF;
      WIDTH    0;
      HEIGHT   0;
      PANEL;
      VISIBLE  .F.;
      ON PAINT Sumatra_FrameRedraw(cPanel)
    END WINDOW

    HMG_ChangeWindowStyle(GetFormHandle(cPanel), WS_SIZEBOX, NIL, .F., .F.)
    ChangeWindowMessageFilter(GetFormHandle(cPanel), 74 /*WM_COPYDATA*/, 1 /*MSGFLT_ALLOW*/)
  ENDIF

RETURN nPanel


FUNCTION PanelName(nPanel)

  IF ! HB_IsNumeric(nPanel)
    nPanel := saTab[PdfView.Tabs.VALUE]
  ENDIF

  IF (nPanel > 0) .and. (nPanel <= Len(saPanel))
    RETURN "P" + HB_NtoS(nPanel)
  ENDIF

RETURN ""


FUNCTION PanelShow(nPanel, lShow)
  LOCAL nHWnd := GetFormHandle(PanelName(nPanel))

  IF lShow
    ShowWindow(nHWnd)
  ELSE
    SetWindowPos(nHWnd, 0, 0xFFFF, 0xFFFF, 0, 0, 0x94 /*SWP_HIDEWINDOW|SWP_NOACTIVATE|SWP_NOZORDER*/)
  ENDIF

RETURN NIL


FUNCTION TabNew(nPanel)
  LOCAL nTab := Len(saTab) + 1

  aAdd(saTab, nPanel)
  PdfView.Tabs.AddPage(nTab, "")

  PdfView.Tabs.VALUE   := nTab
  PdfView.Tabs.VISIBLE := (nTab > 1)

RETURN NIL


FUNCTION TabChange(nTabNew)
  LOCAL nTabCurr
  LOCAL lFileFocus

  IF Len(saTab) > 1
    nTabCurr := PdfView.Tabs.VALUE

    IF nTabNew == 0 //last tab
      nTabNew := Len(saTab)
    ELSEIF nTabNew == -1 //next tab
      IF nTabCurr == Len(saTab)
        nTabNew := 1
      ELSE
        nTabNew := nTabCurr + 1
      ENDIF
    ELSEIF nTabNew == -2 //previous tab
      IF nTabCurr == 1
        nTabNew := Len(saTab)
      ELSE
        nTabNew := nTabCurr - 1
      ENDIF
    ENDIF

    IF (nTabNew != nTabCurr) .and. (nTabNew <= Len(saTab))
      lFileFocus := (GetFocus() == PdfView.Files.HANDLE)
      PdfView.Tabs.VALUE := nTabNew

      PanelShow(saTab[nTabCurr], .F.)
      PanelShow(saTab[nTabNew],  .T.)
      PdfViewResize(.F.)
      StatusSetFile()

      IF slTabGoToFile
        FileGoTo()
      ENDIF

      SetFocus(If(lFileFocus, PdfView.Files.HANDLE, Sumatra_FrameHandle(PanelName(saTab[nTabNew]))))
    ENDIF
  ENDIF

RETURN NIL


FUNCTION TabClose(nTab)
  LOCAL lFileFocus := (GetFocus() == PdfView.Files.HANDLE)
  LOCAL nTabCurr   := PdfView.Tabs.VALUE
  LOCAL cPanel

  IF ! HB_IsNumeric(nTab)
    nTab := nTabCurr
  ENDIF

  cPanel := PanelName(saTab[nTab])
  saPanel[saTab[nTab]] := .T.

  IF nTab == nTabCurr
    GetSumatraSettings()
  ENDIF

  RecentAdd(Sumatra_FileName(cPanel), Sumatra_PageNumber(cPanel))
  Sumatra_FileClose(cPanel, .T.)

  IF Len(saTab) > 1
    IF nTab == nTabCurr
      PanelShow(saTab[nTab], .F.)

      IF nTab == Len(saTab)
        --nTabCurr
      ENDIF
    ELSEIF nTab < nTabCurr
      --nTabCurr
    ENDIF

    HB_aDel(saTab, nTab, .T.)
    PdfView.Tabs.DeletePage(nTab)
    PdfView.Tabs.VALUE := nTabCurr

    PanelShow(saTab[nTabCurr], .T.)

    IF slTabGoToFile
      FileGoTo()
    ENDIF
  ENDIF

  PdfView.Tabs.VISIBLE := (Len(saTab) > 1)

  PdfViewResize(.F.)
  StatusSetFile()

  IF lFileFocus .or. (Sumatra_FrameHandle(PanelName(saTab[nTabCurr])) == 0)
    PdfView.Files.SETFOCUS
  ELSE
    SetFocus(Sumatra_FrameHandle(PanelName(saTab[nTabCurr])))
  ENDIF

RETURN NIL


FUNCTION TabCloseAllEmpty()
  LOCAL n

  FOR n := 1 TO Len(saTab)
    IF Sumatra_FrameHandle(PanelName(saTab[n])) == 0
      TabClose(n)
    ENDIF
  NEXT

RETURN NIL


FUNCTION FileChooseDir()
  LOCAL nRow := GetProperty(PanelName(), "ROW")
  LOCAL nCol := GetProperty(PanelName(), "COL")
  LOCAL cDir

  ClientToScreen(PdfView.HANDLE, @nCol, @nRow)

  cDir := BrowseForFolder(CRLF + LangStr(LS_ChooseDir, .T.) + ":", HB_BitOr(BIF_NEWDIALOGSTYLE, BIF_NONEWFOLDERBUTTON), NIL, NIL, scFileDir, nRow, nCol)

  IF ! Empty(cDir)
    cDir := DirSepAdd(cDir)

    IF HMG_StrCmp(cDir, scFileDir, .F.) != 0
      scFileDir := cDir
      FileListRefresh("")
    ENDIF
  ENDIF

RETURN NIL


FUNCTION FileListRefresh(cFileSel)
  LOCAL nSel := 1
  LOCAL aFile
  LOCAL nLen
  LOCAL n

  EnableWindowRedraw(PdfView.Files.HANDLE, .F.)
  PdfView.Files.DELETEALLITEMS

  IF ! Empty(scFileDir)
    IF VolSerial(HB_ULeft(scFileDir, 3)) == -1
      cFileSel  := HB_ULeft(scFileDir, 2)
      scFileDir := ""
    ELSE
      DO WHILE ! HB_DirExists(scFileDir)
        cFileSel  := ""
        scFileDir := DirParent(scFileDir)
      ENDDO
    ENDIF
  ENDIF

  IF Empty(scFileDir)
    FOR n := 65 TO 90 //from "A" to "Z"
      IF IsDisk(Chr(n))
        PdfView.Files.AddItem({Chr(n) + ":", 0, "", "", "DK"})

        IF (Chr(n) + ":") == cFileSel
          nSel := PdfView.Files.ITEMCOUNT
        ENDIF
      ENDIF
    NEXT

  ELSE
    aFile := Directory(scFileDir + "*.*", "DHS")

    FOR n := Len(aFile) TO 1 STEP -1
      IF "D" $ aFile[n][F_ATTR]
        IF (aFile[n][F_NAME] == ".") .or. (aFile[n][F_NAME] == "..")
          HB_aDel(aFile, n, .T.)
        ENDIF
      ELSEIF HMG_StrCmp(HB_FNameExt(aFile[n][F_NAME]), ".pdf", .F.) != 0
        HB_aDel(aFile, n, .T.)
      ENDIF
    NEXT

    aSort(aFile, NIL, NIL, ;
          {|a1, a2|
            IF ("D" $ a1[F_ATTR]) .and. (! ("D" $ a2[F_ATTR]))
              RETURN .T.
            ELSEIF (! ("D" $ a1[F_ATTR])) .and. ("D" $ a2[F_ATTR])
              RETURN .F.
            ENDIF
            RETURN (HMG_StrCmp(a1[F_NAME], a2[F_NAME], .F.) < 0)
          })

    HB_aIns(aFile, 1, {"..", 0, "", "", "D"}, .T.)

    nLen := Len(aFile)

    FOR n := 1 TO nLen
      IF "D" $ aFile[n][F_ATTR]
        aFile[n][F_NAME] := "<" + aFile[n][F_NAME] + ">"
      ENDIF

      PdfView.Files.AddItem(aFile[n])

      IF aFile[n][F_NAME] == cFileSel
        nSel := n
      ENDIF
    NEXT
  ENDIF

  EnableWindowRedraw(PdfView.Files.HANDLE, .T., .T.)

  PdfView.Files.VALUE := nSel
  PdfView.Files.ColumnWIDTH(1) := PdfView.Files.WIDTH - 4 - If(PdfView.Files.ITEMCOUNT > ListViewGetCountPerPage(PdfView.Files.HANDLE), GetVScrollBarWidth(), 0)
  PdfView.Files.Header(1) := If(Empty(scFileDir), LangStr(LS_Disk), scFileDir)

RETURN NIL


FUNCTION FileGoTo()
  LOCAL cFile
  LOCAL cDir
  LOCAL nCount
  LOCAL lOpenAtOnce
  LOCAL n

  IF ! saPanel[saTab[PdfView.Tabs.VALUE]]
    cFile := Sumatra_FileName(PanelName())

    IF HB_FileExists(cFile)
      cDir  := HB_fNameDir(cFile)
      cFile := HB_fNameNameExt(cFile)

      lOpenAtOnce  := slOpenAtOnce
      slOpenAtOnce := .F.

      IF HMG_StrCmp(cDir, scFileDir, .F.) == 0
        nCount := PdfView.Files.ITEMCOUNT

        FOR n := 1 TO nCount
          IF HMG_StrCmp(cFile, PdfView.Files.CellEx(n, F_NAME), .F.) == 0
            PdfView.Files.VALUE := n
            EXIT
          ENDIF
        NEXT
      ELSE
        scFileDir := cDir
        FileListRefresh(cFile)
      ENDIF

      slOpenAtOnce := lOpenAtOnce
    ENDIF
  ENDIF

RETURN NIL


FUNCTION FileOpen(cFile, lAtPage, lNewTab)
  LOCAL lRecentFile := .F.
  LOCAL nPage
  LOCAL nPos
  LOCAL lFileFocus
  LOCAL cSumatraExe
  LOCAL cRecentFile
  LOCAL nRecentPage
  LOCAL nPanel

  IF HB_IsChar(cFile)
    lRecentFile := .T.
  ELSE
    cFile := PdfView.Files.CellEx(PdfView.Files.VALUE, F_NAME)

    IF "D" $ PdfView.Files.CellEx(PdfView.Files.VALUE, F_ATTR)
      IF "K" $ PdfView.Files.CellEx(PdfView.Files.VALUE, F_ATTR)
        IF VolSerial(DirSepAdd(cFile)) == -1
          MsgWin(cFile + CRLF + CRLF + LangStr(LS_NoDisk))
        ELSE
          scFileDir := DirSepAdd(cFile)
          cFile     := ""
        ENDIF
      ELSE
        cFile := HB_USubstr(cFile, 2, HMG_Len(cFile) - 2)

        IF cFile == ".."
          IF HMG_Len(scFileDir) == 3
            cFile := DirSepDel(scFileDir)
          ELSE
            cFile := "<" + HB_FNameNameExt(DirSepDel(scFileDir)) + ">"
          ENDIF
          scFileDir := DirParent(scFileDir)
        ELSE
          scFileDir := DirSepAdd(scFileDir + cFile)
          cFile    := ""
        ENDIF
      ENDIF

      FileListRefresh(cFile)
      RETURN NIL
    ENDIF

    cFile := scFileDir + cFile
  ENDIF

  IF HB_IsLogical(lAtPage) .and. lAtPage
    nPage := InputPageNum()

    IF HB_IsNIL(nPage)
      RETURN NIL
    ENDIF
  ELSE
    IF HMG_StrCmp(cFile, Sumatra_FileName(PanelName()), .F.) == 0
      nPage := Sumatra_PageNumber(PanelName())
    ELSE
      nPos  := HB_aScan(saRecent, {|aFile| HMG_StrCmp(aFile[1], cFile, .F.) == 0})
      nPage := If(nPos > 0, saRecent[nPos][2], 1)
    ENDIF
  ENDIF

  GetSumatraSettings()

  lFileFocus  := (GetFocus() == PdfView.Files.HANDLE)
  cSumatraExe := "SumatraPDF.exe"

  IF HB_IsLogical(lNewTab) .and. lNewTab
    nPanel := PanelNew()
  ELSE
    nPanel := saTab[PdfView.Tabs.VALUE]
  ENDIF

  IF nPanel == saTab[PdfView.Tabs.VALUE]
    cRecentFile := Sumatra_FileName(PanelName())
    nRecentPage := Sumatra_PageNumber(PanelName())
  ELSE
    PanelShow(nPanel, .T.)
  ENDIF

  nPage := Sumatra_FileOpen(PanelName(nPanel), cFile, nPage, snZoom, slBookmarks, slToolbar, scLang, If(Empty(scSumatraDir), HB_DirBase(), scSumatraDir) + cSumatraExe)

  SWITCH nPage
    CASE -1
      //MsgWin("Panel window is not defined!")
      EXIT
    CASE -2
      IF nPanel != saTab[PdfView.Tabs.VALUE]
        PanelShow(nPanel, .F.)
      ENDIF

      MsgWin(If(Empty(scSumatraDir), HB_DirBase(), DirSepAdd(TrueName(scSumatraDir))) + cSumatraExe + CRLF + CRLF + LangStr(LS_NoFile) + CRLF + LangStr(LS_SetPathTo) + ": " + cSumatraExe + ".")
      InputToolDir(1)
      EXIT
    CASE -3
      IF nPanel != saTab[PdfView.Tabs.VALUE]
        PanelShow(nPanel, .F.)
      ENDIF

      MsgWin(cFile + CRLF + CRLF + LangStr(LS_NoFile) + If(lRecentFile, "", CRLF + LangStr(LS_ListRefresh)))

      IF ! lRecentFile
        FileListRefresh("")
      ENDIF
      EXIT
    OTHERWISE
      saPanel[nPanel] := .F.

      IF nPanel == saTab[PdfView.Tabs.VALUE]
        RecentAdd(cRecentFile, nRecentPage)
      ELSE
        PanelShow(NIL, .F.)
        TabNew(nPanel)
        PdfViewResize(.F.)
      ENDIF

      StatusSetFile()
      SetFocus(If(lFileFocus, PdfView.Files.HANDLE, Sumatra_FrameHandle(PanelName())))
  ENDSWITCH

RETURN NIL


/*
  SessionOpen(nAction, [aFiles])
  nAction:
    1 - on start - open files passed from commad line
    2 - open files passed via WM_COPYDATA (aFiles parameter is required)
    3 - on start - restore last session
    4 - open last session when program already is running
*/
FUNCTION SessionOpen(nAction, aFiles)
  LOCAL cSumatraDir := If(Empty(scSumatraDir), HB_DirBase(), scSumatraDir)
  LOCAL cSumatraExe := "SumatraPDF.exe"
  LOCAL nCount      := 0
  LOCAL lFileFocus  := (GetFocus() == PdfView.Files.HANDLE)
  LOCAL nTabCurr    := PdfView.Tabs.VALUE
  LOCAL nPanel
  LOCAL nStart
  LOCAL nEnd
  LOCAL cFile
  LOCAL nPage
  LOCAL nPos
  LOCAL n

  IF (nAction == 4) .and. (! HB_FileExists(cSumatraDir + cSumatraExe))
    MsgWin(cSumatraDir + cSumatraExe + CRLF + CRLF + LangStr(LS_NoFile) + CRLF + LangStr(LS_SetPathTo) + ": " + cSumatraExe + ".")
    InputToolDir(1)
    RETURN .F.
  ENDIF

  GetSumatraSettings()

  IF (nAction == 1)
    nStart := 1
    nEnd   := HB_ArgC()
  ELSEIF (nAction == 2)
    nStart := 2
    nEnd   := Len(aFiles)
  ELSEIF (nAction == 3) .and. slSessionRest .or. (nAction == 4)
    nStart := 2
    nEnd   := Len(saSession)
  ELSE
    nStart := 1
    nEnd   := 0
  ENDIF

  FOR n := nStart TO nEnd
    nPanel := PanelNew()

    IF nPanel != saTab[PdfView.Tabs.VALUE]
      PanelShow(nPanel, .T.)
    ENDIF

    IF (nAction == 1)
      cFile := HB_ArgV(n)

      IF HB_ULeft(cFile, 1) == "\"
        cFile := HB_ULeft(scDirStart, 2) + cFile
      ELSEIF ! (HB_USubstr(cFile, 2, 1) == ":")
        cFile := scDirStart + cFile
      ENDIF

      nPos  := HB_aScan(saRecent, {|aFile| HMG_StrCmp(aFile[1], cFile, .F.) == 0})
      nPage := If(nPos > 0, saRecent[nPos][2], 1)
    ELSEIF (nAction == 2)
      cFile := aFiles[n]

      IF HB_ULeft(cFile, 1) == "\"
        cFile := HB_ULeft(aFiles[1], 2) + cFile
      ELSEIF ! (HB_USubstr(cFile, 2, 1) == ":")
        cFile := aFiles[1] + cFile
      ENDIF

      nPos  := HB_aScan(saRecent, {|aFile| HMG_StrCmp(aFile[1], cFile, .F.) == 0})
      nPage := If(nPos > 0, saRecent[nPos][2], 1)
    ELSE
      cFile := saSession[n][1]
      nPage := saSession[n][2]
    ENDIF

    IF Sumatra_FileOpen(PanelName(nPanel), cFile, nPage, snZoom, slBookmarks, slToolbar, scLang, cSumatraDir + cSumatraExe) >= 0
      ++nCount
      saPanel[nPanel] := .F.

      IF nPanel != saTab[PdfView.Tabs.VALUE]
        PanelShow(NIL, .F.)
        TabNew(nPanel)
        PdfViewResize(.F.)
      ENDIF

      StatusSetFile()

      IF (nAction > 2) .and. (nCount <= saSession[1][2])
        nTabCurr := PdfView.Tabs.VALUE
      ENDIF
    ELSE
      IF nPanel != saTab[PdfView.Tabs.VALUE]
        PanelShow(nPanel, .F.)
      ENDIF
    ENDIF
  NEXT

  IF (nAction > 2) .and. (nCount > 0) .and. (nTabCurr != PdfView.Tabs.VALUE)
    TabChange(nTabCurr)
  ELSEIF slTabGoToFile
    FileGoTo()
  ENDIF

  IF (nAction == 2) .or. (nAction == 4)
    IF lFileFocus .or. (Sumatra_FrameHandle(PanelName()) == 0)
      PdfView.Files.SETFOCUS
    ELSE
      SetFocus(Sumatra_FrameHandle(PanelName()))
    ENDIF
  ENDIF

RETURN (nCount > 0)


FUNCTION SessionReopen()
  LOCAL lFileFocus := (GetFocus() == PdfView.Files.HANDLE)
  LOCAL nTab       := 1
  LOCAL nTabCurr   := PdfView.Tabs.VALUE
  LOCAL nCount     := Len(saTab)
  LOCAL cFile
  LOCAL nPage
  LOCAL cPanel
  LOCAL n

  FOR n := 1 TO nCount
    TabChange(nTab)

    cPanel := PanelName()
    cFile  := Sumatra_FileName(cPanel)

    IF ! Empty(cFile)
      nPage := Sumatra_PageNumber(cPanel)

      GetSumatraSettings()

      nPage := Sumatra_FileOpen(cPanel, cFile, nPage, snZoom, slBookmarks, slToolbar, scLang, If(Empty(scSumatraDir), HB_DirBase(), scSumatraDir) + "SumatraPDF.exe")

      IF nPage >= 0
        ++nTab
      ELSE
        TabClose(nTab)

        IF nTab < nTabCurr
          --nTabCurr
        ENDIF
      ENDIF
    ENDIF
  NEXT

  IF nTabCurr > Len(saTab)
    nTabCurr := Len(saTab)
  ENDIF

  TabChange(nTabCurr)
  StatusSetPage(.T.)

  IF lFileFocus .or. (Sumatra_FrameHandle(PanelName()) == 0)
    PdfView.Files.SETFOCUS
  ELSE
    SetFocus(Sumatra_FrameHandle(PanelName()))
  ENDIF

RETURN NIL


FUNCTION RecentAdd(cFile, nPage)
  LOCAL nPos

  IF (snRecentAmount > 0) .and. (! Empty(cFile)) .and. (! Empty(nPage))
    nPos := HB_aScan(saRecent, {|aFile| HMG_StrCmp(aFile[1], cFile, .F.) == 0})

    IF nPos > 0
      HB_aDel(saRecent, nPos, .T.)
    ENDIF

    HB_aIns(saRecent, 1, {cFile, nPage}, (Len(saRecent) < snRecentAmount))
  ENDIF

RETURN NIL


FUNCTION StatusSetFile()
  LOCAL cFile := Sumatra_FileName(PanelName())

  IF Empty(cFile)
    PdfView.TITLE := "PdfView"
    PdfView.Tabs.Caption(PdfView.Tabs.VALUE) := ""
    PdfView.STATUSBAR.Item(1) := ""
  ELSE
    PdfView.TITLE := "PdfView: " + HB_fNameNameExt(cFile)
    PdfView.Tabs.Caption(PdfView.Tabs.VALUE) := HB_fNameName(cFile)
    PdfView.STATUSBAR.Item(1) := cFile
  ENDIF

RETURN NIL


FUNCTION StatusSetPage(lForce)
  STATIC nPage   := 0
  STATIC nCount  := 0
  LOCAL  nPage1  := Sumatra_PageNumber(PanelName())
  LOCAL  nCount1 := Sumatra_PageCount(PanelName())

  IF lForce .or. (nPage != nPage1) .or. (nCount != nCount1)
    nPage  := nPage1
    nCount := nCount1

    PdfView.STATUSBAR.Item(2) := If(nCount == 0, "", LangStr(LS_Page, .T.) + ": " + HB_NtoS(nPage) + "/" + HB_NtoS(nCount))
  ENDIF

  RELEASE MEMORY

RETURN NIL


FUNCTION SetZoom(nZoomNew)

  IF Sumatra_PageCount(PanelName()) > 0
    snZoom := nZoomNew
    Sumatra_Zoom(PanelName(), snZoom)
  ENDIF

RETURN NIL


FUNCTION GetSumatraSettings()
  LOCAL cPanel := PanelName()
  LOCAL nHToolbar

  IF Sumatra_FrameHandle(cPanel) != 0
    IF Sumatra_PageCount(cPanel) > 0
      nHToolbar := Sumatra_ToolbarHandle(cPanel)

      IF HB_BitAnd(SendMessage(nHToolbar, 1042 /*TB_GETSTATE*/, 3027 /*IDT_VIEW_FIT_PAGE*/, 0), 0x01 /*TBSTATE_CHECKED*/) != 0
        snZoom := 2
      ELSEIF HB_BitAnd(SendMessage(nHToolbar, 1042 /*TB_GETSTATE*/, 3026 /*IDT_VIEW_FIT_WIDTH*/, 0), 0x01 /*TBSTATE_CHECKED*/) != 0
        snZoom := 4
      ENDIF
    ENDIF

    IF Sumatra_BookmarksExist(cPanel)
      slBookmarks := Sumatra_Bookmarks(cPanel)
    ENDIF

    slToolbar := Sumatra_Toolbar(cPanel)
  ENDIF

RETURN NIL


FUNCTION InputPageNum()
  LOCAL nPage

  DEFINE WINDOW InputPage;
    WIDTH  230 + GetSystemMetrics(7 /*SM_CXFIXEDFRAME*/) * 2;
    HEIGHT  74 + GetSystemMetrics(4 /*SM_CYCAPTION*/) + GetSystemMetrics(8 /*SM_CYFIXEDFRAME*/) * 2;
    TITLE  LangStr(LS_OpenFilePage);
    MODAL;
    NOSIZE

    DEFINE LABEL PageNumber
      ROW       13
      COL       45
      WIDTH     90
      HEIGHT    13
      ALIGNMENT RIGHT
      VALUE     LangStr(LS_PageNum) + ":"
    END LABEL

    DEFINE TEXTBOX Number
      ROW          10
      COL         140
      WIDTH        40
      HEIGHT       21
      VALUE         1
      MAXLENGTH     5
      RIGHTALIGN  .T.
      DATATYPE    NUMERIC
      ONGOTFOCUS  PostMessage(InputPage.Open.HANDLE, 244 /*BM_SETSTYLE*/, 1 /*BS_DEFPUSHBUTTON*/, 1)
      ONLOSTFOCUS SendMessage(InputPage.Open.HANDLE, 244 /*BM_SETSTYLE*/, 0 /*BS_PUSHBUTTON*/, 1)
      ONENTER     ((nPage := InputPage.Number.VALUE), InputPage.RELEASE)
    END TEXTBOX

    DEFINE BUTTON Open
      ROW     41
      COL     40
      WIDTH   70
      HEIGHT  23
      CAPTION LangStr(LS_Open)
      ACTION  ((nPage := InputPage.Number.VALUE), InputPage.RELEASE)
    END BUTTON

    DEFINE BUTTON Cancel
      ROW      41
      COL     120
      WIDTH    70
      HEIGHT   23
      CAPTION LangStr(LS_Cancel)
      ACTION  InputPage.RELEASE
    END BUTTON
  END WINDOW

  SendMessage(InputPage.Open.HANDLE, 244 /*BM_SETSTYLE*/, 1 /*BS_DEFPUSHBUTTON*/, 1)
  C_Center(InputPage.HANDLE, GetActiveWindow())
  EventCreate({ || If(LoWord(EventWPARAM()) == 2 /*IDCANCEL*/, InputPage.RELEASE, NIL) }, InputPage.HANDLE, 273 /*WM_COMMAND*/)

  slFileRefresh := .F.

  InputPage.ACTIVATE

  slFileRefresh := .T.

  IF HB_IsNumeric(nPage) .and. (nPage < 1)
    nPage := 1
  ENDIF

RETURN nPage


FUNCTION InputToolDir(nTool)
  LOCAL nRow := GetProperty(PanelName(), "ROW")
  LOCAL nCol := GetProperty(PanelName(), "COL")
  LOCAL cDir

  ClientToScreen(PdfView.HANDLE, @nCol, @nRow)

  DEFINE WINDOW InputDir;
    ROW    nRow;
    COL    nCol;
    WIDTH  250 + GetSystemMetrics(7 /*SM_CXFIXEDFRAME*/) * 2;
    HEIGHT  74 + GetSystemMetrics(4 /*SM_CYCAPTION*/) + GetSystemMetrics(8 /*SM_CYFIXEDFRAME*/) * 2;
    TITLE  LangStr(If(nTool == 1, LS_SumatraDir, LS_PDFtkDir), .T.);
    MODAL;
    NOSIZE

    DEFINE TEXTBOX Dir
      ROW         10
      COL         10
      WIDTH       230
      HEIGHT      21
      VALUE       If(nTool == 1, scSumatraDir, scPDFtkDir)
      MAXLENGTH   260
      DATATYPE    CHARACTER
      ONGOTFOCUS  PostMessage(InputDir.OK.HANDLE, 244 /*BM_SETSTYLE*/, 1 /*BS_DEFPUSHBUTTON*/, 1)
      ONLOSTFOCUS SendMessage(InputDir.OK.HANDLE, 244 /*BM_SETSTYLE*/, 0 /*BS_PUSHBUTTON*/,    1)
      ONENTER     ((cDir := InputDir.Dir.VALUE), InputDir.RELEASE)
    END TEXTBOX

    DEFINE BUTTON Browse
      ROW     41
      COL     10
      WIDTH   70
      HEIGHT  23
      CAPTION LangStr(LS_Browse) + "..."
      ACTION  BrowseForToolDir(nTool)
    END BUTTON

    DEFINE BUTTON OK
      ROW     41
      COL     90
      WIDTH   70
      HEIGHT  23
      CAPTION LangStr(LS_OK)
      ACTION  ((cDir := InputDir.Dir.VALUE), InputDir.RELEASE)
    END BUTTON

    DEFINE BUTTON Cancel
      ROW     41
      COL     170
      WIDTH   70
      HEIGHT  23
      CAPTION LangStr(LS_Cancel)
      ACTION  InputDir.RELEASE
    END BUTTON
  END WINDOW

  SendMessage(InputDir.OK.HANDLE, 244 /*BM_SETSTYLE*/, 1 /*BS_DEFPUSHBUTTON*/, 1)
  EventCreate({ || If(LoWord(EventWPARAM()) == 2 /*IDCANCEL*/, InputDir.RELEASE, NIL) }, InputDir.HANDLE, 273 /*WM_COMMAND*/)

  slFileRefresh := .F.

  InputDir.ACTIVATE

  slFileRefresh := .T.

  IF HB_IsString(cDir)
    IF nTool == 1
      scSumatraDir := DirSepAdd(cDir)
    ELSE
      scPDFtkDir := DirSepAdd(cDir)
    ENDIF
  ENDIF

RETURN NIL


FUNCTION BrowseForToolDir(nTool)
  LOCAL nRow := InputDir.Dir.ROW + InputDir.Dir.HEIGHT
  LOCAL nCol := InputDir.Dir.COL
  LOCAL cDir := DirSepAdd(TrueName(InputDir.Dir.VALUE))

  ClientToScreen(InputDir.HANDLE, @nCol, @nRow)

  SendMessage(InputDir.OK.HANDLE,     244 /*BM_SETSTYLE*/, 0 /*BS_PUSHBUTTON*/, 1)
  SendMessage(InputDir.Cancel.HANDLE, 244 /*BM_SETSTYLE*/, 0 /*BS_PUSHBUTTON*/, 1)

  IF ! HB_DirExists(cDir)
    cDir := HB_DirBase()
  ENDIF

  cDir := BrowseForFolder(CRLF + LangStr(If(nTool == 1, LS_SelSumatraDir, LS_SelPDFtkDir)) + ":", HB_BitOr(BIF_NEWDIALOGSTYLE, BIF_NONEWFOLDERBUTTON), NIL, NIL, cDir, nRow + 3, nCol)

  IF ! Empty(cDir)
    HMG_EditControlSetSel(InputDir.Dir.HANDLE, 0, -1)
    HMG_EditControlReplaceSel(InputDir.Dir.HANDLE, .T., DirSepAdd(cDir))
    InputDir.Dir.SETFOCUS
    SendMessage(InputDir.Browse.HANDLE, 244 /*BM_SETSTYLE*/, 0 /*BS_PUSHBUTTON*/, 1)
  ENDIF

RETURN NIL


FUNCTION AboutPdfView()
  LOCAL nHDC

  DEFINE WINDOW About;
    WIDTH  270 + GetSystemMetrics(7 /*SM_CXFIXEDFRAME*/) * 2;
    HEIGHT 308 + GetSystemMetrics(4 /*SM_CYCAPTION*/) + GetSystemMetrics(8 /*SM_CYFIXEDFRAME*/) * 2;
    TITLE  LangStr(LS_AboutPdfView);
    MODAL;
    NOSIZE

    DEFINE LABEL Label1
      ROW       10
      HEIGHT    32
      FONTNAME  "Times New Roman"
      FONTSIZE  22
      FONTBOLD  .T.
      VALUE     "PdfView"
    END LABEL

    DEFINE LABEL Label2
      ROW    25
      HEIGHT 13
      VALUE  "v." + PDFVIEW_VERSION + " (" + LTrim(Str(HB_Version(HB_VERSION_BITWIDTH))) + "-bit)"
    END LABEL

    DEFINE LABEL Label3
      ROW    50
      WIDTH  About.CLIENTAREAWIDTH
      HEIGHT 13
      VALUE  LangStr(LS_PdfViewUsing) + ":"
      ALIGNMENT CENTER
    END LABEL

    DEFINE HYPERLINK Link1
      ROW        70
      HEIGHT     13
      HANDCURSOR .T.
      VALUE      "SumatraPDF"
    END HYPERLINK

    DEFINE HYPERLINK Link2
      ROW        90
      HEIGHT     13
      HANDCURSOR .T.
      VALUE      "PDFtk Server"
      ADDRESS    "http://www.pdftk.com"
    END HYPERLINK

    DEFINE LABEL Label4
      ROW        120
      COL        -2
      WIDTH      274
      HEIGHT     82
      CLIENTEDGE .T.
      VALUE      CRLF + Chr(9) + ;
                 LangStr(LS_DevelopedIn) + ":"  + CRLF + Chr(9) + Space(5) + ;
                 _HMG_VERSION_NUMBER_           + CRLF + Chr(9) + Space(5) + ;
                 HB_Version(HB_VERSION_HARBOUR) + CRLF + Chr(9) + Space(5) + ;
                 HB_Version(HB_VERSION_COMPILER)
    END LABEL

    DEFINE LABEL Label5
      ROW        200
      COL        -2
      WIDTH      274
      HEIGHT     43
      CLIENTEDGE .T.
      VALUE      CRLF + LangStr(LS_Author) + ": Krzysztof Janicki aka KDJ"
    END LABEL

    DEFINE LABEL Label6
      ROW        241
      COL        -2
      WIDTH      274
      HEIGHT     69
      CLIENTEDGE .T.
      VALUE      CRLF + Chr(9) + ;
                 LangStr(LS_Translators) + ":"       + CRLF + Chr(9) + Space(5) + ;
                 "Pablo César (Portuguese, Spanish)" + CRLF + Chr(9) + Space(5) + ;
                 "Krzysztof Janicki (Polish, Russian)"
    END LABEL
  END WINDOW

  nHDC := GetDC(About.HANDLE)

  About.Label1.WIDTH := GetTextWidth(nHDC, About.Label1.VALUE, GetWindowFont(About.Label1.HANDLE))
  About.Label2.WIDTH := GetTextWidth(nHDC, About.Label2.VALUE, GetWindowFont(About.Label2.HANDLE))
  About.Label1.COL   := Int((About.CLIENTAREAWIDTH - About.Label1.WIDTH - About.Label2.WIDTH - 10) / 2)
  About.Label2.COL   := About.Label1.COL + About.Label1.WIDTH + 10

  About.Link1.WIDTH := GetTextWidth(nHDC, About.Link1.VALUE, GetWindowFont(About.Link1.HANDLE))
  About.Link1.COL   := Int((About.CLIENTAREAWIDTH - About.Link1.WIDTH) / 2)
  About.Link2.WIDTH := GetTextWidth(nHDC, About.Link2.VALUE, GetWindowFont(About.Link2.HANDLE))
  About.Link2.COL   := Int((About.CLIENTAREAWIDTH - About.Link2.WIDTH) / 2)

  ReleaseDC(About.HANDLE, nHDC)

  //bug in HMG, if protocol is "https" it opens "mailto"
  _HMG_SYSDATA[6][GetControlIndex("Link1", "About")] := {|| ShellExecute(0, "open", "rundll32.exe", "url.dll, FileProtocolHandler " + "https://www.sumatrapdfreader.org", , 1 /*SW_SHOWNORMAL*/)}

  HMG_ChangeWindowStyle(About.Label5.HANDLE, 0x0001 /*SS_CENTER*/, NIL, .F., .F.)

  EventCreate({ || If(LoWord(EventWPARAM()) == 2 /*IDCANCEL*/, About.RELEASE, NIL) }, About.HANDLE, 273 /*WM_COMMAND*/)

  slFileRefresh := .F.

  About.CenterIn(PdfView)
  About.ACTIVATE

  slFileRefresh := .T.

RETURN NIL

FUNCTION MsgWin(aMsg, cTitle)
  LOCAL cMsg := ""
  LOCAL nHDC
  LOCAL nHFont
  LOCAL cLine
  LOCAL nLaW, nLaH
  LOCAL nCAW

  IF HB_IsString(aMsg)
    aMsg := HB_aTokens(aMsg, .T. /*lEOL*/)
  ENDIF

  IF ! HB_IsString(cTitle)
    cTitle := "PdfView"
  ENDIF

  IF IsWindowVisible(GetMainFormHandle())
    DEFINE WINDOW MsgWnd;
      MODAL;
      NOSIZE
  ELSE
    DEFINE WINDOW MsgWnd;
      CHILD;
      NOMINIMIZE;
      NOMAXIMIZE;
      NOSIZE
  ENDIF

    DEFINE LABEL MsgText
      ROW 10
    END LABEL

    DEFINE BUTTON OK
      WIDTH   80
      HEIGHT  23
      CAPTION LangStr(LS_OK)
      ACTION  MsgWnd.RELEASE
    END BUTTON
  END WINDOW

  nHDC   := GetDC(MsgWnd.MsgText.HANDLE)
  nHFont := GetWindowFont(MsgWnd.MsgText.HANDLE)
  nLaW   := 100

  FOR EACH cLine IN aMsg
    nLaW := Max(nLaW, GetTextWidth(nHDC, cLine, nHFont))
    cMsg += cLine + CRLF
  NEXT

  nLaH := GetTextHeight(nHDC, aMsg[1], nHFont) * Len(aMsg)

  ReleaseDC(MsgWnd.MsgText.HANDLE, nHDC)

  nCAW := nLaW + 20

  MsgWnd.WIDTH  := nCAW + (MsgWnd.WIDTH - MsgWnd.CLIENTAREAWIDTH)
  MsgWnd.HEIGHT := nLaH + 53 + (MsgWnd.HEIGHT - MsgWnd.CLIENTAREAHEIGHT)
  MsgWnd.TITLE  := cTitle

  MsgWnd.MsgText.COL     := Int((nCAW - nLaW) / 2)
  MsgWnd.MsgText.WIDTH   := nLaW
  MsgWnd.MsgText.HEIGHT  := nLaH
  MsgWnd.MsgText.CAPTION := cMsg

  MsgWnd.OK.ROW := nLaH + 20
  MsgWnd.OK.COL := Int((nCAW - 80) / 2)

  C_Center(MsgWnd.HANDLE, GetActiveWindow())

  IF MsgWnd.ROW < 0
    MsgWnd.ROW := 0
  ENDIF

  EventCreate({ || If(LoWord(EventWPARAM()) == 2 /*IDCANCEL*/, MsgWnd.RELEASE, NIL) }, MsgWnd.HANDLE, 273 /*WM_COMMAND*/)

  slFileRefresh := .F.

  MsgWnd.ACTIVATE

  slFileRefresh := .T.

RETURN NIL


FUNCTION RecentFiles()
  LOCAL lFileFocus := (GetFocus() == PdfView.Files.HANDLE)
  LOCAL aFile
  LOCAL nHDC

  DEFINE WINDOW Recent;
    ROW    0;
    COL    0;
    WIDTH  snRecent_W;
    HEIGHT snRecent_H;
    TITLE  LangStr(LS_RecentFiles, .T.);
    MODAL;
    ON PAINT   PaintSizeGrip(Recent.HANDLE);
    ON SIZE    RecentResize();
    ON RELEASE ((snRecent_W := Recent.WIDTH), (snRecent_H := Recent.HEIGHT), RecentAmount(.F.))

    DEFINE GRID Files
      ROW            10
      COL            10
      SHOWHEADERS    .F.
      WIDTHS         {0}
      CELLNAVIGATION .F.
      ONDBLCLICK     RecentFileOpen((GetKeyState(VK_CONTROL) < 0), (GetKeyState(VK_SHIFT) < 0))
      ONCHANGE       RecentCount()
      ONKEY          RecentFilesOnKey()
    END GRID

    DEFINE CHECKBOX Names
      COL      15
      WIDTH    90
      HEIGHT   16
      CAPTION  LangStr(LS_OnlyNames)
      VALUE    slRecentNames
      ONCHANGE RecentNames()
    END CHECKBOX

    DEFINE LABEL Count
      WIDTH     60
      HEIGHT    13
      ALIGNMENT RIGHT
    END LABEL

    DEFINE LABEL AmountLabel
      COL    10
      HEIGHT 13
      VALUE  LangStr(LS_FilesAmount) + ":"
    END LABEL

    DEFINE SPINNER Amount
      ROW         0
      COL         0
      WIDTH       40
      HEIGHT      21
      RANGEMIN    0
      RANGEMAX    999
      VALUE       snRecentAmount
      ONLOSTFOCUS RecentAmount(.T.)
    END SPINNER

    DEFINE BUTTON Open
      WIDTH         90
      HEIGHT        23
      CAPTION       LangStr(LS_Open)
      PICTURE       "BUTTONARROW"
      PICTALIGNMENT RIGHT
      ACTION        RecentOpenMenu()
    END BUTTON

    DEFINE BUTTON Remove
      WIDTH         90
      HEIGHT        23
      CAPTION       LangStr(LS_Remove)
      PICTURE       "ButtonArrow"
      PICTALIGNMENT RIGHT
      ACTION        RecentRemoveMenu()
    END BUTTON
  END WINDOW

  ListView_ChangeExtendedStyle(Recent.Files.HANDLE, LVS_EX_INFOTIP)

  FOR EACH aFile IN saRecent
    Recent.Files.AddItem({If(slRecentNames, HB_fNameName(aFile[1]), aFile[1])})
  NEXT

  Recent.Files.VALUE := 1

  nHDC := GetDC(Recent.AmountLabel.HANDLE)
  Recent.AmountLabel.WIDTH := GetTextWidth(nHDC, LangStr(LS_FilesAmount) + ":", GetWindowFont(Recent.AmountLabel.HANDLE))
  ReleaseDC(Recent.AmountLabel.HANDLE, nHDC)

  RecentCount()
  RecentButtonsEnable()
  RecentResize()

  slFileRefresh := .F.

  Recent.CenterIn(PdfView)
  Recent.ACTIVATE

  slFileRefresh := .T.

  SetFocus(If(lFileFocus, PdfView.Files.HANDLE, Sumatra_FrameHandle(PanelName())))

RETURN NIL


FUNCTION RecentNames()
  LOCAL n

  slRecentNames := Recent.Names.VALUE

  FOR n := 1 TO Len(saRecent)
    Recent.Files.CellEx(n, 1) := If(slRecentNames, HB_fNameName(saRecent[n][1]), saRecent[n][1])
  NEXT

RETURN NIL


FUNCTION RecentCount()

  Recent.Count.VALUE := HB_NtoS(Recent.Files.VALUE) + "/" + HB_NtoS(Recent.Files.ITEMCOUNT)

RETURN NIL


FUNCTION RecentAmount(lRefreshList)
  LOCAL n

  snRecentAmount := Recent.Amount.VALUE

  IF snRecentAmount < Len(saRecent)
    IF lRefreshList
      EnableWindowRedraw(Recent.Files.HANDLE, .F.)

      FOR n := Len(saRecent) TO (snRecentAmount + 1) STEP -1
        Recent.Files.DeleteItem(n)
      NEXT

      IF Recent.Files.VALUE == 0
        Recent.Files.VALUE := snRecentAmount
      ENDIF

      EnableWindowRedraw(Recent.Files.HANDLE, .T., .T.)
      RecentCount()
      RecentButtonsEnable()
    ENDIF

    aSize(saRecent, snRecentAmount)
  ENDIF

RETURN NIL


FUNCTION RecentOpenMenu()
  LOCAL aRect  := {0, 0, 0, 0}
  LOCAL nHMenu := CreatePopupMenu()
  LOCAL nCmd

  GetWindowRect(Recent.Open.HANDLE, aRect)

  AppendMenuString(nHMenu, 1, LangStr(LS_OpenCurTab) + e"\tEnter")
  AppendMenuString(nHMenu, 2, LangStr(LS_OpenNewTab) + e"\tShift+Enter")
  AppendMenuSeparator(nHMenu)
  AppendMenuString(nHMenu, 3, LangStr(LS_OpenPage)    + e"...\tCtrlt+Enter")
  AppendMenuString(nHMenu, 4, LangStr(LS_OpenPageTab) + e"...\tCtrlt+Shift+Enter")

  nCmd := TrackPopupMenu2(nHMenu, 0x0188 /*TPM_NONOTIFY|TPM_RETURNCMD|TPM_RIGHTALIGN*/, aRect[4], aRect[3], Recent.HANDLE)

  DestroyMenu(nHMenu)

  SWITCH nCmd
    CASE 1
      RecentFileOpen(.F., .F.)
      EXIT
    CASE 2
      RecentFileOpen(.F., .T.)
      EXIT
    CASE 3
      RecentFileOpen(.T., .F.)
      EXIT
    CASE 4
      RecentFileOpen(.T., .T.)
      EXIT
  ENDSWITCH

RETURN NIL


FUNCTION RecentRemoveMenu()
  LOCAL aRect  := {0, 0, 0, 0}
  LOCAL nHMenu := CreatePopupMenu()
  LOCAL nCmd

  GetWindowRect(Recent.Remove.HANDLE, aRect)

  AppendMenuString(nHMenu, 1, LangStr(LS_Selected) + e"\tShift+Del")
  AppendMenuString(nHMenu, 2, LangStr(LS_NonExistent))
  AppendMenuString(nHMenu, 3, LangStr(LS_All))

  nCmd := TrackPopupMenu2(nHMenu, 0x0188 /*TPM_NONOTIFY|TPM_RETURNCMD|TPM_RIGHTALIGN*/, aRect[4], aRect[3], Recent.HANDLE)

  DestroyMenu(nHMenu)

  IF nCmd > 0
    RecentFileRemove(nCmd)
  ENDIF

RETURN NIL


FUNCTION RecentResize()
  LOCAL nRecentCW := Recent.CLIENTAREAWIDTH
  LOCAL nRecentCH := Recent.CLIENTAREAHEIGHT

  Recent.Files.WIDTH     := nRecentCW - 20
  Recent.Files.HEIGHT    := nRecentCH - 74
  Recent.Names.ROW       := nRecentCH - 59
  Recent.Count.ROW       := nRecentCH - 59
  Recent.Count.COL       := nRecentCW - Recent.Count.WIDTH - 15
  Recent.AmountLabel.ROW := nRecentCH - 29
  Recent.Amount.ROW      := nRecentCH - 32
  Recent.Amount.COL      := Recent.AmountLabel.WIDTH + 13
  Recent.Open.ROW        := nRecentCH - 33
  Recent.Open.COL        := nRecentCW - (90 + 10) * 2
  Recent.Remove.ROW      := nRecentCH - 33
  Recent.Remove.COL      := nRecentCW - (90 + 10)

  Recent.Files.ColumnWIDTH(1) := nRecentCW - 20 - 4 - If(Recent.Files.ITEMCOUNT > ListViewGetCountPerPage(Recent.Files.HANDLE), GetVScrollBarWidth(), 0)

RETURN NIL


FUNCTION RecentFilesOnKey()

  IF HMG_GetLastVirtualKeyDown() == VK_DELETE
    HMG_CleanLastVirtualKeyDown()

    IF (GetKeyState(VK_CONTROL) >= 0) .and. (GetKeyState(VK_SHIFT) < 0) .and. (GetKeyState(VK_MENU) >= 0)
      RecentFileRemove(1)
    ENDIF
  ENDIF

RETURN NIL


FUNCTION RecentButtonsEnable()
  LOCAL lEnable := (Recent.Files.ITEMCOUNT > 0)
  LOCAL nHFocus := GetFocus()

  IF (! lEnable) .and. ((nHFocus == Recent.Open.HANDLE) .or. (nHFocus == Recent.Remove.HANDLE))
    Recent.Files.SETFOCUS
  ENDIF

  Recent.Open.ENABLED   := lEnable
  Recent.Remove.ENABLED := lEnable

RETURN NIL


FUNCTION RecentFileOpen(lAtPage, lNewTab)
  LOCAL nPos := Recent.Files.VALUE
  LOCAL cFile
  LOCAL n

  IF nPos > 0
    cFile := saRecent[nPos][1]

    FileOpen(cFile, lAtPage, lNewTab)

    IF slTabGoToFile
      FileGoTo()
    ENDIF

    EnableWindowRedraw(Recent.Files.HANDLE, .F.)

    IF Recent.Files.ITEMCOUNT < Len(saRecent)
      Recent.Files.AddItem({""})
    ENDIF

    FOR n := 1 TO Len(saRecent)
      Recent.Files.CellEx(n, 1) := If(slRecentNames, HB_fNameName(saRecent[n][1]), saRecent[n][1])

      IF HMG_StrCmp(cFile, saRecent[n][1], .F.) == 0
        nPos := n
      ENDIF
    NEXT

    Recent.Files.VALUE := nPos

    EnableWindowRedraw(Recent.Files.HANDLE, .T., .T.)
    RecentCount()
  ENDIF

RETURN NIL


/*
  nAction:
    1 - remove selected
    2 - remove non-existent
    3 - remove all
*/
FUNCTION RecentFileRemove(nAction)
  LOCAL nPos := Recent.Files.VALUE
  LOCAL n

  IF nPos > 0
    IF nAction == 1
      Recent.Files.DeleteItem(nPos)
      HB_aDel(saRecent, nPos, .T.)

      IF nPos > Recent.Files.ITEMCOUNT
        Recent.Files.VALUE := Recent.Files.ITEMCOUNT
      ELSE
        Recent.Files.VALUE := nPos
      ENDIF
    ELSEIF nAction == 2
      EnableWindowRedraw(Recent.Files.HANDLE, .F.)

      FOR n := Len(saRecent) TO 1 STEP -1
        IF ! HB_FileExists(saRecent[n][1])
          Recent.Files.DeleteItem(n)
          HB_aDel(saRecent, n, .T.)
        ENDIF
      NEXT

      IF nPos > Recent.Files.ITEMCOUNT
        Recent.Files.VALUE := Recent.Files.ITEMCOUNT
      ELSE
        Recent.Files.VALUE := nPos
      ENDIF

      EnableWindowRedraw(Recent.Files.HANDLE, .T., .T.)
    ELSEIF nAction == 3
      Recent.Files.DELETEALLITEMS
      aSize(saRecent, 0)
    ENDIF

    RecentCount()
    RecentButtonsEnable()
  ENDIF

RETURN NIL


FUNCTION PdfSplit()
  LOCAL nPages := Sumatra_PageCount(PanelName())
  LOCAL cPDFtkName
  LOCAL cFileName
  LOCAL cFile
  LOCAL cExt
  LOCAL nRGB
  LOCAL aColor
  LOCAL nRow
  LOCAL nCol
  LOCAL nPagesLen
  LOCAL cOutFilesText

  IF nPages < 2
    RETURN NIL
  ENDIF

  cPDFtkName := If(Empty(scPDFtkDir), HB_DirBase(), DirSepAdd(TrueName(scPDFtkDir))) + "PDFtk.exe"

  IF ! HB_FileExists(cPDFtkName)
    MsgWin(cPDFtkName + CRLF + CRLF + LangStr(LS_NoFile) + CRLF + LangStr(LS_SetPathTo) + ": PDFtk.exe.")
    InputToolDir(2)
    RETURN NIL
  ENDIF

  cFileName := Sumatra_FileName(PanelName())

  IF ! HB_FileExists(cFileName)
    MsgWin(cFileName + CRLF + CRLF + LangStr(LS_NoFile))
    RETURN NIL
  ENDIF

  cFile  := HB_fNameName(cFileName)
  cExt   := HB_fNameExt(cFileName)
  nRGB   := GetSysColor(15 /*COLOR_BTNFACE*/)
  aColor := {GetRed(nRGB), GetGreen(nRGB), GetBlue(nRGB)}
  nRow   := GetProperty(PanelName(), "ROW")
  nCol   := GetProperty(PanelName(), "COL")

  ClientToScreen(PdfView.HANDLE, @nCol, @nRow)

  nPagesLen     = Len(HB_NtoS(nPages))
  cOutFilesText = cFile + "_" + StrZero(1, nPagesLen) + cExt + CRLF + cFile + "_" + StrZero(2, nPagesLen) + cExt

  IF nPages > 2
    IF nPages < 5
      cOutFilesText += CRLF + cFile + "_" + StrZero(3, nPagesLen) + cExt
      IF nPages == 4
        cOutFilesText += CRLF + cFile + "_" + StrZero(4, nPagesLen) + cExt
      ENDIF
    ELSE
      cOutFilesText += CRLF + "..." + CRLF + cFile + "_" + StrZero(nPages, nPagesLen) + cExt
    ENDIF
  ENDIF

  DEFINE WINDOW Split;
    ROW    nRow;
    COL    nCol;
    WIDTH  460 + GetSystemMetrics(7 /*SM_CXFIXEDFRAME*/) * 2;
    HEIGHT 205 + GetSystemMetrics(4 /*SM_CYCAPTION*/) + GetSystemMetrics(8 /*SM_CYFIXEDFRAME*/) * 2;
    TITLE  LangStr(LS_SplitPages);
    MODAL;
    NOSIZE

    DEFINE TEXTBOX File
      ROW               10
      COL               10
      WIDTH             440
      HEIGHT            21
      VALUE             cFile + cExt
      DATATYPE          CHARACTER
      READONLY          .T.
      DISABLEDBACKCOLOR aColor
      ONGOTFOCUS        PostMessage(Split.OK.HANDLE, 244 /*BM_SETSTYLE*/, 1 /*BS_DEFPUSHBUTTON*/, 1)
      ONLOSTFOCUS       SendMessage(Split.OK.HANDLE, 244 /*BM_SETSTYLE*/, 0 /*BS_PUSHBUTTON*/, 1)
      ONENTER           If(IsWindowActive(Split), PDFtk_Burst(cPDFtkName, cFileName, nPages, nPagesLen), NIL)
    END TEXTBOX

    DEFINE LABEL OutDirLabel
      ROW    41
      COL    10
      WIDTH  440
      HEIGHT 13
      VALUE  LangStr(LS_OutputDir) + ":"
    END LABEL

    DEFINE TEXTBOX OutDir
      ROW               56
      COL               10
      WIDTH             420
      HEIGHT            21
      VALUE             HB_fNameDir(cFileName)
      DATATYPE          CHARACTER
      READONLY          .T.
      DISABLEDBACKCOLOR aColor
      ONGOTFOCUS        PostMessage(Split.OK.HANDLE, 244 /*BM_SETSTYLE*/, 1 /*BS_DEFPUSHBUTTON*/, 1)
      ONLOSTFOCUS       SendMessage(Split.OK.HANDLE, 244 /*BM_SETSTYLE*/, 0 /*BS_PUSHBUTTON*/, 1)
      ONENTER           If(IsWindowActive(Split), PDFtk_Burst(cPDFtkName, cFileName, nPages, nPagesLen), NIL)
    END TEXTBOX

    DEFINE BUTTON OutDirButton
      ROW     55
      COL     430
      WIDTH   20
      HEIGHT  23
      CAPTION "..."
      ACTION  PdfSplit_OutDir()
    END BUTTON

    DEFINE LABEL OutFilesLabel
      ROW    87
      COL    10
      WIDTH  440
      HEIGHT 13
      VALUE  LangStr(LS_TargetFiles) + " (" + HB_NtoS(nPages) + "):"
    END LABEL

    DEFINE EDITBOX OutFiles
      ROW               102
      COL               10
      WIDTH             440
      HEIGHT            60
      VALUE             cOutFilesText
      READONLY          .T.
      DISABLEDBACKCOLOR aColor
      VSCROLLBAR        .F.
      HSCROLLBAR        .F.
      ONGOTFOCUS        PostMessage(Split.OK.HANDLE, 244 /*BM_SETSTYLE*/, 1 /*BS_DEFPUSHBUTTON*/, 1)
      ONLOSTFOCUS       SendMessage(Split.OK.HANDLE, 244 /*BM_SETSTYLE*/, 0 /*BS_PUSHBUTTON*/, 1)
    END EDITBOX

    DEFINE LABEL Wait
      ROW       175
      COL       10
      WIDTH     280
      HEIGHT    13
      VISIBLE   .F.
      FONTCOLOR {255, 0, 0}
    END LABEL

    DEFINE BUTTON OK
      ROW     172
      COL     300
      WIDTH   70
      HEIGHT  23
      CAPTION LangStr(LS_OK)
      ACTION  PDFtk_Burst(cPDFtkName, cFileName, nPages, nPagesLen)
    END BUTTON

    DEFINE BUTTON Cancel
      ROW     172
      COL     380
      WIDTH   70
      HEIGHT  23
      CAPTION LangStr(LS_Cancel)
      ACTION  Split.RELEASE
    END BUTTON
  END WINDOW

  SendMessage(Split.OK.HANDLE, 244 /*BM_SETSTYLE*/, 1 /*BS_DEFPUSHBUTTON*/, 1)
  EventCreate({ || If(LoWord(EventWPARAM()) == 2 /*IDCANCEL*/, Split.RELEASE, NIL) }, Split.HANDLE, 273 /*WM_COMMAND*/)
  EventCreate({ || If(EventWPARAM() == 27 /*VK_ESCAPE*/, (Split.OK.SETFOCUS, Split.RELEASE), If(EventWPARAM() == 13 /*VK_RETURN*/, PDFtk_Burst(cPDFtkName, cFileName, nPages, nPagesLen), NIL)) }, Split.OutFiles.HANDLE, 256 /*WM_KEYDOWN*/)

  slFileRefresh := .F.

  Split.ACTIVATE

  slFileRefresh := .T.

RETURN NIL


FUNCTION PdfSplit_OutDir()
  LOCAL nRow := Split.OutDir.ROW + Split.OutDir.HEIGHT
  LOCAL nCol := Split.OutDir.COL
  LOCAL cDir

  ClientToScreen(Split.HANDLE, @nCol, @nRow)

  cDir := BrowseForFolder(CRLF + LangStr(LS_ChooseDir, .T.) + ":", BIF_NEWDIALOGSTYLE, NIL, NIL, Split.OutDir.VALUE, nRow + 3, nCol)

  IF ! Empty(cDir)
    Split.OutDir.VALUE := DirSepAdd(cDir)
  ENDIF

RETURN NIL


/*
  PDFtk v2.02 doesn't support Unicode file names.
  So we use a copy of files with ANSI names.
*/
FUNCTION PDFtk_Burst(cPDFtkName, cFileName, nPages, nPagesLen)
  LOCAL cFile
  LOCAL cExt
  LOCAL cOutDir
  LOCAL cTmpDir
  LOCAL cTmpFileFull
  LOCAL cPass
  LOCAL cStdErr
  LOCAL nExitCode
  LOCAL cPage
  LOCAL lOpenAtOnce
  LOCAL n

  IF ! HB_FileExists(cFileName)
    MsgWin(cFileName + CRLF + CRLF + LangStr(LS_NoFile))
    RETURN NIL
  ENDIF

  Split.Wait.VALUE := LangStr(LS_Wait) + ": " + LangStr(LS_CopySource) + " (1/1)"
  Split.Wait.SHOW

  cExt         := HB_fNameExt(cFileName)
  cTmpDir      := TmpDirCreate()
  cTmpFileFull := cTmpDir + "a" + cExt
  cPass        := ""

  HB_fCopy(cFileName, cTmpFileFull)

  Split.Wait.VALUE := LangStr(LS_Wait) + ": " + LangStr(LS_PDFtkWorking)

  DO WHILE .T.
    nExitCode := HB_ProcessRun('"' + cPDFtkName + '" "' + cTmpFileFull + '" input_pw "' + cPass + '" burst output "' + cTmpDir + 'a_%0' + HB_NtoS(Len(HB_NtoS(nPages))) + 'd' + cExt + '"', NIL, NIL, @cStdErr, .T.)

    IF (nExitCode == 1) .and. ("OWNER PASSWORD REQUIRED" $ cStdErr)
      Split.Wait.HIDE

      cPass := InputPassword(ClientToScreenRow(Split.HANDLE, Split.File.ROW + Split.File.HEIGHT) + 3, ClientToScreenCol(Split.HANDLE, Split.File.COL), cPass)

      IF HB_IsString(cPass)
        Split.Wait.SHOW
      ELSE
        EXIT
      ENDIF
    ELSE
      IF nExitCode < 0
        Split.Wait.HIDE
        MsgWin(LangStr(LS_CantRunPDFtk), LangStr(LS_PDFtkError))
      ELSEIF nExitCode > 0
        Split.Wait.HIDE
        MsgWin(cStdErr, LangStr(LS_PDFtkError))
      ENDIF

      EXIT
    ENDIF
  ENDDO

  IF nExitCode == 0
    lOpenAtOnce  := slOpenAtOnce
    slOpenAtOnce := .F.
    cFile        := HB_fNameName(cFileName)
    cOutDir      := Split.OutDir.VALUE
    scFileDir    := cOutDir

    HB_DirBuild(cOutDir)

    FOR n := 1 TO nPages
      cPage := "_" + StrZero(n, nPagesLen)
      Split.Wait.VALUE := LangStr(LS_Wait) + ": " + LangStr(LS_CopyTarget) + " (" + HB_NtoS(n) + "/" + HB_NtoS(nPages) + ")"
      HB_fCopy(cTmpDir + "a" + cPage + cExt, cOutDir + cFile + cPage + cExt)
    NEXT

    Split.RELEASE
    FileListRefresh(cFile + "_" + StrZero(1, nPagesLen) + cExt)
    PdfView.Files.SETFOCUS

    slOpenAtOnce := lOpenAtOnce
  ENDIF

  HB_DirRemoveAll(cTmpDir)

RETURN NIL


FUNCTION TmpDirCreate()
  LOCAL cTmpDir := GetTempDir()
  LOCAL cName   := HB_fNameName(GetProgramFileName())
  LOCAL nCount  := 0

  HB_DirBuild(cTmpDir)

  DO WHILE HB_fNameExists(cTmpDir + cName + "_" + HB_NtoS(nCount))
    ++nCount
  ENDDO

  HB_DirCreate(cTmpDir += cName + "_" + HB_NtoS(nCount) + "\")

RETURN cTmpDir


FUNCTION InputPassword(nRow, nCol, cPassInp)
  LOCAL cPassOut

  DEFINE WINDOW InputPass;
    ROW    nRow;
    COL    nCol;
    WIDTH  250 + GetSystemMetrics(7 /*SM_CXFIXEDFRAME*/) * 2;
    HEIGHT 100 + GetSystemMetrics(4 /*SM_CYCAPTION*/) + GetSystemMetrics(8 /*SM_CYFIXEDFRAME*/) * 2;
    TITLE  LangStr(LS_OwnerPassReq);
    MODAL;
    NOSIZE

    DEFINE TEXTBOX Pass
      ROW         10
      COL         10
      WIDTH       230
      HEIGHT      21
      VALUE       cPassInp
      MAXLENGTH   32
      DATATYPE    CHARACTER
      PASSWORD    (! slPassShow)
      ONGOTFOCUS  PostMessage(InputPass.OK.HANDLE, 244 /*BM_SETSTYLE*/, 1 /*BS_DEFPUSHBUTTON*/, 1)
      ONLOSTFOCUS SendMessage(InputPass.OK.HANDLE, 244 /*BM_SETSTYLE*/, 0 /*BS_PUSHBUTTON*/, 1)
      ONENTER     If(IsWindowActive(InputPass), ((cPassOut := InputPass.Pass.VALUE), InputPass.RELEASE), NIL)
    END TEXTBOX

    DEFINE CHECKBOX PassShow
      ROW         41
      COL         10
      WIDTH       120
      HEIGHT      16
      VALUE       slPassShow
      CAPTION     LangStr(LS_ShowPass)
      ONGOTFOCUS  PostMessage(InputPass.OK.HANDLE, 244 /*BM_SETSTYLE*/, 1 /*BS_DEFPUSHBUTTON*/, 1)
      ONLOSTFOCUS SendMessage(InputPass.OK.HANDLE, 244 /*BM_SETSTYLE*/, 0 /*BS_PUSHBUTTON*/, 1)
      ONENTER     If(IsWindowActive(InputPass), ((cPassOut := InputPass.Pass.VALUE), InputPass.RELEASE), NIL)
      ONCHANGE    ((slPassShow := ! slPassShow), SendMessage(InputPass.Pass.HANDLE, 0x00CC /*EM_SETPASSWORDCHAR*/, If(slPassShow, 0, 0x25CF), 0), InputPass.Pass.REDRAW)
    END CHECKBOX

    DEFINE BUTTON OK
      ROW     67
      COL     50
      WIDTH   70
      HEIGHT  23
      CAPTION LangStr(LS_OK)
      ACTION  ((cPassOut := InputPass.Pass.VALUE), InputPass.RELEASE)
    END BUTTON

    DEFINE BUTTON Cancel
      ROW     67
      COL     130
      WIDTH   70
      HEIGHT  23
      CAPTION LangStr(LS_Cancel)
      ACTION  InputPass.RELEASE
    END BUTTON
  END WINDOW

  SendMessage(InputPass.OK.HANDLE, 244 /*BM_SETSTYLE*/, 1 /*BS_DEFPUSHBUTTON*/, 1)
  EventCreate({ || If(IsWindowActive(InputPass) .and. (LoWord(EventWPARAM()) == 2 /*IDCANCEL*/), InputPass.RELEASE, NIL) }, InputPass.HANDLE, 273 /*WM_COMMAND*/)

  InputPass.ACTIVATE

RETURN cPassOut


FUNCTION MainEventHandler(nHWnd, nMsg, nWParam, nLParam)
  LOCAL nHWndFrom
  LOCAL cPanel
  LOCAL nMaxY
  LOCAL aRect1, aRect2
  LOCAL nCol

  IF nHWnd == PdfView.HANDLE
    SWITCH nMsg
      CASE 36 /*WM_GETMINMAXINFO*/
        SetMinMaxTrackSize(nLParam, 550, 300)
        EXIT

      CASE 274 /*WM_SYSCOMMAND*/
        IF HB_BitAnd(nWParam, 0xFFF0) == 0xF100 /*SC_KEYMENU*/
          //for Win-10
          IF GetFocus() != PdfView.Files.HANDLE
            snHFocusMenu := GetFocus()
            PdfView.Files.SETFOCUS
            PostMessage(nHWnd, nMsg, nWParam, nLParam)
            RETURN 1
          ENDIF

          IF GetMenu(nHWnd) == 0
            SetMenu(nHWnd, snHMenuMain)
          ELSEIF ! slMenuBar
            SetMenu(nHWnd, 0)
            RETURN 1
          ENDIF

          PdfView.PdfTimer.ENABLED := .F.
        ENDIF
        RETURN 0

      CASE 278 /*WM_INITMENU*/
        slMenuActive := .T.
        MenuCommandsEnable()
        SetOnKey(.F.)
        RETURN 0

      CASE 530 /*WM_EXITMENULOOP*/
        slMenuActive := .F.
        SetOnKey(.T.)

        //for Win-10
        IF IsWindowEnabled(snHFocusMenu)
          SetFocus(snHFocusMenu)
          snHFocusMenu := NIL
        ENDIF

        IF ! slMenuBar
          SetMenu(nHWnd, 0)
        ENDIF

        PdfView.PdfTimer.ENABLED := .T.
        RETURN 0

      CASE 78 /*WM_NOTIFY*/
        nHWndFrom := GetHWNDFrom(nLParam)

        IF nHWndFrom == PdfView.Files.HANDLE
          SWITCH GetNotifyCode(nLParam)
            CASE -2 /*NM_CLICK*/
            CASE -5 /*NM_RCLICK*/
            CASE -6 /*NM_RDBLCLK*/
              IF (PdfView.Files.VALUE == 0) .and. (PdfView.Files.CELLROWFOCUSED > 0)
                PdfView.Files.VALUE := PdfView.Files.CELLROWFOCUSED
              ENDIF
              EXIT
            CASE -3 /*NM_DBLCLK*/
              IF (PdfView.Files.VALUE == 0) .and. (PdfView.Files.CELLROWFOCUSED > 0)
                PdfView.Files.VALUE := PdfView.Files.CELLROWFOCUSED
              ELSE
                FileOpen(NIL, (GetKeyState(VK_CONTROL) < 0), (GetKeyState(VK_SHIFT) < 0))
              ENDIF
              RETURN 1
          ENDSWITCH

        ELSEIF nHWndFrom == PdfView.Tabs.HANDLE
          SWITCH GetNotifyCode(nLParam)
            CASE -551 /*TCN_SELCHANGE*/
              PanelShow(saTab[PdfView.Tabs.VALUE], .T.)
              PdfViewResize(.F.)
              StatusSetFile()

              IF slTabGoToFile
                FileGoTo()
              ENDIF

              EXIT
            CASE -552 /*TCN_SELCHANGING*/
              PanelShow(saTab[PdfView.Tabs.VALUE], .F.)
              RETURN 0
          ENDSWITCH
        ENDIF
        EXIT

      CASE 123 /*WM_CONTEXTMENU*/
        IF nWParam == PdfView.Files.HANDLE
          FileMenu(HiWord(nLParam), LoWord(nLParam))
        ENDIF
        EXIT

      CASE 74 /*WM_COPYDATA*/
        IF GetCopyDataAction(nLParam) == 1
          SessionOpen(2, HB_aTokens(GetCopyDataString(nLParam), Chr(9)))
        ENDIF
        EXIT
    ENDSWITCH

  ELSEIF nHWnd == GetFormHandle(cPanel := PanelName())
    SWITCH nMsg
      CASE 36 /*WM_GETMINMAXINFO*/
        nMaxY := PdfView.CLIENTAREAHEIGHT - If(PdfView.Tabs.VISIBLE, PdfView.Tabs.HEIGHT, 0) - GetWindowHeight(PdfView.STATUSBAR.HANDLE)
        SetMinMaxTrackSize(nLParam, 200, nMaxY, PdfView.CLIENTAREAWIDTH - 100, nMaxY)
        EXIT

      CASE 532 /*WM_SIZING*/
        aRect1 := Array(4)
        aRect2 := Array(4)

        GetWindowRect(nHWnd, aRect1)
        GetRectArray(nLParam, aRect2)

        SWITCH nWParam
          CASE 1 /*WMSZ_LEFT*/
          CASE 4 /*WMSZ_TOPLEFT*/
          CASE 7 /*WMSZ_BOTTOMLEFT*/
            IF aRect1[1] != aRect2[1]
              nCol := ScreenToClientCol(PdfView.HANDLE, aRect2[1])
              PdfView.Files.WIDTH := nCol + 1
              PdfView.Files.ColumnWIDTH(1) := nCol - 4 - If(PdfView.Files.ITEMCOUNT > ListViewGetCountPerPage(PdfView.Files.HANDLE), GetVScrollBarWidth(), 0)
              PdfView.Tabs.COL := nCol
              SetProperty(cPanel, "WIDTH", PdfView.CLIENTAREAWIDTH - nCol + (GetProperty(cPanel, "WIDTH") - GetProperty(cPanel, "CLIENTAREAWIDTH")) / 2)
              Sumatra_FrameAdjust(cPanel)
            ENDIF
            EXIT
          OTHERWISE
            SetRectArray(nLParam, aRect1)
        ENDSWITCH
        EXIT

      CASE 74 /*WM_COPYDATA*/
        IF GetCopyDataAction(nLParam) == 0x4C5255 /*URL*/
          ShellExecute(0, "open", "rundll32.exe", "url.dll, FileProtocolHandler " + GetCopyDataString(nLParam, .T.), NIL, 1 /*SW_SHOWNORMAL*/)
        ENDIF
        EXIT
    ENDSWITCH

  ELSEIF IsWindowDefined(Recent) .and. (nHWnd == Recent.HANDLE)
    SWITCH nMsg
      CASE 36 /*WM_GETMINMAXINFO*/
        SetMinMaxTrackSize(nLParam, 380, 250)
        EXIT

      CASE 78 /*WM_NOTIFY*/
        nHWndFrom := GetHWNDFrom(nLParam)

        IF nHWndFrom == Recent.Files.HANDLE
          SWITCH GetNotifyCode(nLParam)
            CASE -2 /*NM_CLICK*/
            CASE -5 /*NM_RCLICK*/
            CASE -6 /*NM_RDBLCLK*/
              IF (Recent.Files.VALUE == 0) .and. (Recent.Files.CELLROWFOCUSED > 0)
                Recent.Files.VALUE := Recent.Files.CELLROWFOCUSED
              ENDIF
              EXIT
            CASE -3 /*NM_DBLCLK*/
              IF (Recent.Files.VALUE == 0) .and. (Recent.Files.CELLROWFOCUSED > 0)
                Recent.Files.VALUE := Recent.Files.CELLROWFOCUSED
              ELSEIF (Recent.Files.VALUE > 0)
                RecentFileOpen((GetKeyState(VK_CONTROL) < 0), (GetKeyState(VK_SHIFT) < 0))
              ENDIF
              RETURN 1
          ENDSWITCH
        ENDIF
        EXIT

      CASE 273 //WM_COMMAND
        IF LoWord(nWParam) == 2 /*IDCANCEL*/
          Recent.RELEASE
        ENDIF
        EXIT
    ENDSWITCH
  ENDIF

RETURN NIL


FUNCTION IsProgramRunning()
  LOCAL cCurProcName := HMG_Upper(GetProgramFileName())
  LOCAL nCurProcID   := GetCurrentProcessId()
  LOCAL aProcID      := EnumProcessesID()
  LOCAL nPID
  LOCAL aHWnd
  LOCAL nWinPID
  LOCAL nHMainWnd
  LOCAL cFiles
  LOCAL n

  FOR n := 1 TO Len(aProcID)
    IF (HMG_Upper(GetProcessFullName(aProcID[n])) == cCurProcName) .and. (aProcID[n] != nCurProcID)
      nPID := aProcID[n]
      EXIT
    ENDIF
  NEXT

  IF HB_IsNumeric(nPID)
    aHWnd := EnumWindows()

    FOR n := 1 TO Len(aHWnd)
      //IF (HMG_Upper(HB_ULeft(GetClassName(aHWnd[n]), 13)) == "_HMG_PDFVIEW_")
      IF IsWindowVisible(aHWnd[n]) .and. (! Empty(GetWindowText(aHWnd[n])))
        GetWindowThreadProcessId(aHWnd[n], NIL, @nWinPID)

        IF nWinPID == nPID
          nHMainWnd := aHWnd[n]
          EXIT
        ENDIF
      ENDIF
    NEXT

    IF HB_IsNumeric(nHMainWnd)
      IF IsMinimized(nHMainWnd)
        Restore(nHMainWnd)
      ELSE
        SetForegroundWindow(nHMainWnd)
      ENDIF

      IF HB_ArgC() > 0
        cFiles := scDirStart + Chr(9)

        FOR n := 1 TO HB_ArgC()
          cFiles += HB_ArgV(n) + If(n == HB_ArgC(), "", Chr(9))
        NEXT

        Send_WM_COPYDATA(nHMainWnd, 1, cFiles)
      ENDIF
    ELSE
      MsgWin(LangStr(LS_AppRunning))
    ENDIF
  ENDIF

RETURN HB_IsNumeric(nPID)


FUNCTION ChangeWindowMessageFilter(nHWnd, nMsg, nAction)
  LOCAL nMajorVer := WinMajorVersionNumber()
  LOCAL nRetVal

  IF nMajorVer >= 6
    IF (nMajorVer == 6) .and. (WinMinorVersionNumber() == 0)
      nRetVal := HMG_CallDLL("User32", NIL, "ChangeWindowMessageFilter", nMsg, nAction)
    ELSE
      nRetVal := HMG_CallDLL("User32", NIL, "ChangeWindowMessageFilterEx", nHWnd, nMsg, nAction, 0)
    ENDIF
  ENDIF

RETURN nRetVal


FUNCTION DirSepAdd(cDir)

  IF (! Empty(cDir)) .and. (! (HB_URight(cDir, 1) == "\"))
    cDir += "\"
  ENDIF

RETURN cDir


FUNCTION DirSepDel(cDir)

  IF HB_URight(cDir, 1) == "\"
    cDir := HB_StrShrink(cDir)
  ENDIF

RETURN cDir


FUNCTION DirParent(cDir)

  cDir := DirSepDel(cDir)

RETURN HB_ULeft(cDir, HB_UTF8RAt("\", cDir))


FUNCTION SettingsRead()
  LOCAL aLine
  LOCAL cLine
  LOCAL cValue
  LOCAL aFile
  LOCAL nPos
  LOCAL nPosS
  LOCAL nPosF
  LOCAL n

  saTab        := {}
  saPanel      := {}
  slMenuActive := .F.

  saSession := {}
  saRecent  := {}

  slPdfView_Max  := .F.
  snPdfView_R    := 100
  snPdfView_C    := 100
  snPdfView_W    := 650
  snPdfView_H    := 450
  snFiles_W      := 150
  snRecent_W     := 380
  snRecent_H     := 250
  slRecentNames  := .F.
  snRecentAmount := 99
  snZoom         := 0
  slMenuBar      := .T.
  slToolbar      := .F.
  slBookmarks    := .T.
  slOpenAtOnce   := .F.
  slSessionRest  := .F.
  slTabGoToFile  := .F.
  slEscExit      := .F.
  slPassShow     := .F.
  scSumatraDir   := ""
  scPDFtkDir     := ""
  scFileDir      := HB_CWD()
  scFileLast     := ""
  scLang         := "en"

  aLine := HB_aTokens(HB_MemoRead(HB_fNameExtSet(GetProcessFullName(), "ini")), CRLF)

  FOR EACH cLine IN aLine
    cLine := AllTrim(cLine)
    nPos  := HB_UAt("=", cLine)

    IF nPos > 0
      cValue := HB_USubstr(cLine, nPos + 1)

      SWITCH HB_ULeft(cLine, nPos - 1)
        CASE "PdfView_Max"  ; slPdfView_Max  := (cValue == "T") ; EXIT
        CASE "PdfView_R"    ; snPdfView_R    := Val(cValue)     ; EXIT
        CASE "PdfView_C"    ; snPdfView_C    := Val(cValue)     ; EXIT
        CASE "PdfView_W"    ; snPdfView_W    := Val(cValue)     ; EXIT
        CASE "PdfView_H"    ; snPdfView_H    := Val(cValue)     ; EXIT
        CASE "Files_W"      ; snFiles_W      := Val(cValue)     ; EXIT
        CASE "Recent_W"     ; snRecent_W     := Val(cValue)     ; EXIT
        CASE "Recent_H"     ; snRecent_H     := Val(cValue)     ; EXIT
        CASE "RecentNames"  ; slRecentNames  := (cValue == "T") ; EXIT
        CASE "RecentAmount" ; snRecentAmount := Val(cValue)     ; EXIT
        CASE "Zoom"         ; snZoom         := Val(cValue)     ; EXIT
        CASE "MenuBar"      ; slMenuBar      := (cValue == "T") ; EXIT
        CASE "Toolbar"      ; slToolbar      := (cValue == "T") ; EXIT
        CASE "Bookmarks"    ; slBookmarks    := (cValue == "T") ; EXIT
        CASE "OpenAtOnce"   ; slOpenAtOnce   := (cValue == "T") ; EXIT
        CASE "RestSession"  ; slSessionRest  := (cValue == "T") ; EXIT
        CASE "TabGoToFile"  ; slTabGoToFile  := (cValue == "T") ; EXIT
        CASE "EscExit"      ; slEscExit      := (cValue == "T") ; EXIT
        CASE "PassShow"     ; slPassShow     := (cValue == "T") ; EXIT
        CASE "SumatraDir"   ; scSumatraDir   := cValue          ; EXIT
        CASE "PDFtkDir"     ; scPDFtkDir     := cValue          ; EXIT
        CASE "FileDir"      ; scFileDir      := cValue          ; EXIT
        CASE "FileLast"     ; scFileLast     := cValue          ; EXIT
        CASE "Lang"         ; scLang         := cValue          ; EXIT
      ENDSWITCH
    ENDIF
  NEXT

  IF snPdfView_W < 550
    snPdfView_W := 550
  ENDIF

  IF snPdfView_H < 300
    snPdfView_H := 300
  ENDIF

  IF snFiles_W < 100
    snFiles_W := 100
  ENDIF

  IF snRecent_W < 380
    snRecent_W := 380
  ENDIF

  IF snRecent_H < 250
    snRecent_H := 250
  ENDIF

  scSumatraDir := DirSepAdd(scSumatraDir)
  scPDFtkDir   := DirSepAdd(scPDFtkDir)

  IF ! Empty(scFileDir)
    IF (VolSerial(HB_ULeft(scFileDir, 3)) == -1) .or. (! HB_DirExists(scFileDir))
      scFileDir := HB_CWD()
    ELSE
      scFileDir := DirSepAdd(scFileDir)
    ENDIF
  ENDIF

  scDirStart := HB_CWD(HB_DirBase())

  IF HB_aScan(LangStr(), {|aLang| aLang[3] == scLang}) == 0
    scLang := "en"
  ENDIF

  aLine := HB_aTokens(HB_MemoRead(HB_fNameExtSet(GetProcessFullName(), "recent")), CRLF)
  nPosS := HB_aScan(aLine, "<session>", NIL, NIL, .T.)
  nPosF := HB_aScan(aLine, "<files>", NIL, NIL, .T.)

  IF nPosS > 0
    nPos := If(nPosS > nPosF, Len(aLine), nPosF - 1)

    FOR n := (nPosS + 1) TO nPos
      IF ! Empty(aLine[n])
        aFile := HB_aTokens(aLine[n], Chr(9))
        aAdd(saSession, {aFile[1], If(Len(aFile) > 1, Val(aFile[2]), 1)})
      ENDIF
    NEXT
  ENDIF

  IF nPosF > 0
    nPos := If(nPosF > nPosS, Len(aLine), nPosS - 1)

    FOR n := (nPosF + 1) TO nPos
      IF ! Empty(aLine[n])
        aFile := HB_aTokens(aLine[n], Chr(9))
        aAdd(saRecent, {aFile[1], If(Len(aFile) > 1, Val(aFile[2]), 1)})
      ENDIF
    NEXT
  ENDIF

RETURN NIL


FUNCTION SettingsWrite()
  LOCAL aWndPos := GetWindowNormalPos(PdfView.HANDLE)
  LOCAL cText   := "<session>" + CRLF
  LOCAL nPanel
  LOCAL cPanel
  LOCAL nPage
  LOCAL cFile
  LOCAL aFile

  GetSumatraSettings()

  IF ! saPanel[saTab[PdfView.Tabs.VALUE]]
    cText += "tab" + Chr(9) + HB_NtoS(PdfView.Tabs.VALUE) + CRLF

    FOR EACH nPanel IN saTab
      cPanel := PanelName(nPanel)
      cFile  := Sumatra_FileName(cPanel)
      nPage  := Sumatra_PageNumber(cPanel)

      IF ! Empty(cFile)
        cText += cFile + Chr(9) + HB_NtoS(nPage) + CRLF

        IF nPage > 0
          RecentAdd(cFile, nPage)
        ENDIF
      ENDIF

      Sumatra_FileClose(cPanel, .F.)
    NEXT
  ENDIF

  cText += "<files>" + CRLF

  FOR EACH aFile IN saRecent
    cText += aFile[1] + Chr(9) + HB_NtoS(aFile[2]) + CRLF
  NEXT

  HB_MemoWrit(HB_fNameExtSet(GetProcessFullName(), "recent"), cText)

  cText := "PdfView_Max="  + If(PdfView.ISMAXIMIZED, "T", "F") + CRLF + ;
           "PdfView_R="    + HB_NtoS(aWndPos[2]) + CRLF + ;
           "PdfView_C="    + HB_NtoS(aWndPos[1]) + CRLF + ;
           "PdfView_W="    + HB_NtoS(aWndPos[3] - aWndPos[1]) + CRLF + ;
           "PdfView_H="    + HB_NtoS(aWndPos[4] - aWndPos[2]) + CRLF + ;
           "Files_W="      + HB_NtoS(PdfView.Files.WIDTH) + CRLF + ;
           "Recent_W="     + HB_NtoS(snRecent_W) + CRLF + ;
           "Recent_H="     + HB_NtoS(snRecent_H) + CRLF + ;
           "RecentNames="  + If(slRecentNames, "T", "F") + CRLF + ;
           "RecentAmount=" + HB_NtoS(snRecentAmount) + CRLF + ;
           "Zoom="         + HB_NtoS(snZoom) + CRLF + ;
           "MenuBar="      + If(slMenuBar, "T", "F") + CRLF + ;
           "Toolbar="      + If(slToolbar, "T", "F") + CRLF + ;
           "Bookmarks="    + If(slBookmarks, "T", "F") + CRLF + ;
           "RestSession="  + If(slSessionRest, "T", "F") + CRLF + ;
           "TabGoToFile="  + If(slTabGoToFile, "T", "F") + CRLF + ;
           "OpenAtOnce="   + If(slOpenAtOnce, "T", "F") + CRLF + ;
           "EscExit="      + If(slEscExit, "T", "F") + CRLF + ;
           "PassShow="     + If(slPassShow, "T", "F") + CRLF + ;
           "SumatraDir="   + scSumatraDir + CRLF + ;
           "PDFtkDir="     + scPDFtkDir + CRLF + ;
           "FileDir="      + scFileDir + CRLF + ;
           "FileLast="     + PdfView.Files.CellEx(PdfView.Files.VALUE, F_NAME) + CRLF + ;
           "Lang="         + scLang + CRLF

  HB_MemoWrit(HB_fNameExtSet(GetProcessFullName(), "ini"), cText)

RETURN NIL


FUNCTION LangStr(nStr, lRemoveAmpersand)
  LOCAL cText

  IF ! HB_IsNumeric(nStr)
    RETURN { ;
             {"English",    "",            "en"}, ;
             {"Polish",     "(Polski)",    "pl"}, ;
             {"Portuguese", "(Português)", "pt"}, ;
             {"Russian",    "(Русский)",   "ru"}, ;
             {"Spanish",    "(Español)",   "es"}  ;
           }
  ENDIF

  cText := ""

  SWITCH scLang
    CASE "pl"
      SWITCH nStr
        CASE LS_File          ; cText := "&Plik"                                       ; EXIT
        CASE LS_OpenCurTab    ; cText := "&Otwórz w bieżącej karcie"                   ; EXIT
        CASE LS_OpenNewTab    ; cText := "Otwórz w &nowej karcie"                      ; EXIT
        CASE LS_OpenPage      ; cText := "Otwórz na &stronie"                          ; EXIT
        CASE LS_OpenPageTab   ; cText := "Otwórz na stronie w nowej &karcie"           ; EXIT
        CASE LS_OpenSession   ; cText := "Otwórz ostatnią s&esję"                      ; EXIT
        CASE LS_RecentFiles   ; cText := "Ostatnio otwarte &pliki"                     ; EXIT
        CASE LS_GoToSubDir    ; cText := "Przejdź do podkatalogu"                      ; EXIT
        CASE LS_GoToParentDir ; cText := "Przejdź do katalogu nadrzędnego"             ; EXIT
        CASE LS_ChooseDir     ; cText := "&Wybierz katalog"                            ; EXIT
        CASE LS_RefreshList   ; cText := "Odśwież &listę"                              ; EXIT
        CASE LS_Exit          ; cText := "&Zakończ"                                    ; EXIT
        CASE LS_Document      ; cText := "&Dokument"                                   ; EXIT
        CASE LS_SaveAs        ; cText := "Z&apisz jako"                                ; EXIT
        CASE LS_Print         ; cText := "&Drukuj"                                     ; EXIT
        CASE LS_Proper        ; cText := "&Właściwości"                                ; EXIT
        CASE LS_Close         ; cText := "&Zamknij"                                    ; EXIT
        CASE LS_SplitPages    ; cText := "Podziel ne pojedyncze strony"                ; EXIT
        CASE LS_ChooseDoc     ; cText := "W&ybierz dokument"                           ; EXIT
        CASE LS_GoToFile      ; cText := "Przejdź do &pliku"                           ; EXIT
        CASE LS_Page          ; cText := "&Strona"                                     ; EXIT
        CASE LS_GoTo          ; cText := "Przejdź &do"                                 ; EXIT
        CASE LS_Prev          ; cText := "Poprz&ednia"                                 ; EXIT
        CASE LS_Next          ; cText := "&Następna"                                   ; EXIT
        CASE LS_First         ; cText := "&Pierwsza"                                   ; EXIT
        CASE LS_Last          ; cText := "&Ostatnia"                                   ; EXIT
        CASE LS_Find          ; cText := "&Znajdź"                                     ; EXIT
        CASE LS_Text          ; cText := "&Tekst"                                      ; EXIT
        CASE LS_PrevOccur     ; cText := "&Poprzednie wystąpienie"                     ; EXIT
        CASE LS_NextOccur     ; cText := "&Następne wystąpienie"                       ; EXIT
        CASE LS_Zoom          ; cText := "&Rozmiar"                                    ; EXIT
        CASE LS_SizeDn        ; cText := "&Zmniejsz"                                   ; EXIT
        CASE LS_SizeUp        ; cText := "&Powiększ"                                   ; EXIT
        CASE LS_ZoomFactor    ; cText := "&Współczynnik powiększenia"                  ; EXIT
        CASE LS_FitPage       ; cText := "&Dopasuj do strony"                          ; EXIT
        CASE LS_ActualSize    ; cText := "&Rozmiar rzeczywisty"                        ; EXIT
        CASE LS_FitWidth      ; cText := "Dopasuj do &szerokości"                      ; EXIT
        CASE LS_Rotate        ; cText := "&Obrót"                                      ; EXIT
        CASE LS_Left          ; cText := "W &lewo"                                     ; EXIT
        CASE LS_Right         ; cText := "W &prawo"                                    ; EXIT
        CASE LS_View          ; cText := "&Widok"                                      ; EXIT
        CASE LS_MenuBar       ; cText := "Pasek &menu"                                 ; EXIT
        CASE LS_Toolbar       ; cText := "Pasek &narzędzi"                             ; EXIT
        CASE LS_Bookmarks     ; cText := "&Zakładki PDF"                               ; EXIT
        CASE LS_Show          ; cText := "&Pokaż"                                      ; EXIT
        CASE LS_ExpandAll     ; cText := "&Rozwiń wszystkie"                           ; EXIT
        CASE LS_CollapseAll   ; cText := "&Zwiń wszystkie"                             ; EXIT
        CASE LS_Settings      ; cText := "&Ustawienia"                                 ; EXIT
        CASE LS_OpenAtOnce    ; cText := "&Otwieraj PDF od razu"                       ; EXIT
        CASE LS_RestSession   ; cText := "&Przywróć ostatnią sesję na starcie"         ; EXIT
        CASE LS_TabGoToFile   ; cText := "&Zmieniając kartę, przejdź do pliku"         ; EXIT
        CASE LS_EscExit       ; cText := "&Esc - kończy program"                       ; EXIT
        CASE LS_SumatraDir    ; cText := "Katalog &SumatraPDF"                         ; EXIT
        CASE LS_PDFtkDir      ; cText := "Katalog &PDFtk"                              ; EXIT
        CASE LS_Language      ; cText := "&Język"                                      ; EXIT
        CASE LS_AboutPdfView  ; cText := "O programie PdfView"                         ; EXIT
        CASE LS_AboutSumatra  ; cText := "O programie SumatraPDF"                      ; EXIT
        CASE LS_AppRunning    ; cText := "Program już jest uruchomiony!"               ; EXIT
        CASE LS_Disk          ; cText := "Dysk"                                        ; EXIT
        CASE LS_NoDisk        ; cText := "Dysk jest niedostępny!"                      ; EXIT
        CASE LS_NoFile        ; cText := "Plik nie istnieje!"                          ; EXIT
        CASE LS_SetPathTo     ; cText := "Ustaw ścieżkę do"                            ; EXIT
        CASE LS_ListRefresh   ; cText := "Lista zostanie odświeżona."                  ; EXIT
        CASE LS_OpenFilePage  ; cText := "Otwórz plik na stronie"                      ; EXIT
        CASE LS_PageNum       ; cText := "Numer strony"                                ; EXIT
        CASE LS_SelSumatraDir ; cText := "Wybierz katalog SumatraPDF"                  ; EXIT
        CASE LS_SelPDFtkDir   ; cText := "Wybierz katalog PDFtk"                       ; EXIT
        CASE LS_OnlyNames     ; cText := "Tylko &nazwy"                                ; EXIT
        CASE LS_FilesAmount   ; cText := "Liczba &plików"                              ; EXIT
        CASE LS_Selected      ; cText := "&Zaznaczony"                                 ; EXIT
        CASE LS_NonExistent   ; cText := "&Nieistniejące"                              ; EXIT
        CASE LS_All           ; cText := "&Wszystkie"                                  ; EXIT
        CASE LS_OutputDir     ; cText := "Katalog docelowy"                            ; EXIT
        CASE LS_TargetFiles   ; cText := "Pliki docelowe"                              ; EXIT
        CASE LS_CantRunPDFtk  ; cText := "Nie można uruchomić PDFtk.exe!"              ; EXIT
        CASE LS_PDFtkError    ; cText := "Błąd PDFtk"                                  ; EXIT
        CASE LS_OwnerPassReq  ; cText := "Wymagane hasło właściciela"                  ; EXIT
        CASE LS_ShowPass      ; cText := "Pokaż hasło"                                 ; EXIT
        CASE LS_Wait          ; cText := "Czekaj"                                      ; EXIT
        CASE LS_PDFtkWorking  ; cText := "PDFtk pracuje"                               ; EXIT
        CASE LS_CopySource    ; cText := "kopiowanie plików źródłowych"                ; EXIT
        CASE LS_CopyTarget    ; cText := "kopiowanie plików docelowych"                ; EXIT
        CASE LS_PdfViewUsing  ; cText := "Podgląd, łączenie i podział PDFów za pomocą" ; EXIT
        CASE LS_DevelopedIn   ; cText := "Opracowany w"                                ; EXIT
        CASE LS_Author        ; cText := "Autor"                                       ; EXIT
        CASE LS_Translators   ; cText := "Tłumacze"                                    ; EXIT
        CASE LS_Open          ; cText := "&Otwórz"                                     ; EXIT
        CASE LS_Remove        ; cText := "&Usuń"                                       ; EXIT
        CASE LS_Browse        ; cText := "&Przeglądaj"                                 ; EXIT
        CASE LS_OK            ; cText := "OK"                                          ; EXIT
        CASE LS_Cancel        ; cText := "&Anuluj"                                     ; EXIT
      ENDSWITCH
      EXIT

    //Portuguese - translated by Pablo César
    CASE "pt"
      SWITCH nStr
        CASE LS_File          ; cText := "Arquivo"                               ; EXIT
          CASE LS_OpenCurTab    ; cText := "&Open in current tab"                  ; EXIT
          CASE LS_OpenNewTab    ; cText := "Open in &new tab"                      ; EXIT
        CASE LS_OpenPage      ; cText := "Abrir arquivo na página"               ; EXIT
          CASE LS_OpenPageTab   ; cText := "Open at page in new &tab"              ; EXIT
          CASE LS_OpenSession   ; cText := "Open last &session"                    ; EXIT
          CASE LS_RecentFiles   ; cText := "Recent &files"                         ; EXIT
          CASE LS_GoToSubDir    ; cText := "Go to subdirectory"                    ; EXIT
          CASE LS_GoToParentDir ; cText := "Go to parent directory"                ; EXIT
          CASE LS_ChooseDir     ; cText := "Choose &directory"                     ; EXIT
        CASE LS_RefreshList   ; cText := "Atualizar"                             ; EXIT
        CASE LS_Exit          ; cText := "Sair"                                  ; EXIT
          CASE LS_Document      ; cText := "&Document"                             ; EXIT
        CASE LS_SaveAs        ; cText := "Salvar como"                           ; EXIT
        CASE LS_Print         ; cText := "Imprimir"                              ; EXIT
        CASE LS_Proper        ; cText := "Propriedades"                          ; EXIT
        CASE LS_Close         ; cText := "Fechar"                                ; EXIT
          CASE LS_SplitPages    ; cText := "Split into single pages"               ; EXIT
          CASE LS_ChooseDoc     ; cText := "Choose &document"                      ; EXIT
          CASE LS_GoToFile      ; cText := "Go to &file"                           ; EXIT
        CASE LS_Page          ; cText := "Página"                                ; EXIT
        CASE LS_GoTo          ; cText := "Ir para"                               ; EXIT
        CASE LS_Prev          ; cText := "Anterior"                              ; EXIT
        CASE LS_Next          ; cText := "Próximo"                               ; EXIT
        CASE LS_First         ; cText := "Primeiro"                              ; EXIT
        CASE LS_Last          ; cText := "Ultimo"                                ; EXIT
        CASE LS_Find          ; cText := "Procurar"                              ; EXIT
        CASE LS_Text          ; cText := "Texto"                                 ; EXIT
        CASE LS_PrevOccur     ; cText := "Ocorrência anterior"                   ; EXIT
        CASE LS_NextOccur     ; cText := "Próxima ocorrência"                    ; EXIT
        CASE LS_Zoom          ; cText := "Visualização"                          ; EXIT
        CASE LS_SizeDn        ; cText := "Diminuir"                              ; EXIT
        CASE LS_SizeUp        ; cText := "Aumentar"                              ; EXIT
        CASE LS_ZoomFactor    ; cText := "Nível de ampliação"                    ; EXIT
        CASE LS_FitPage       ; cText := "Ajuste de página"                      ; EXIT
        CASE LS_ActualSize    ; cText := "Zoom automático"                       ; EXIT
        CASE LS_FitWidth      ; cText := "Ajuste de largura"                     ; EXIT
        CASE LS_Rotate        ; cText := "Girar"                                 ; EXIT
        CASE LS_Left          ; cText := "Para a esquerda"                       ; EXIT
        CASE LS_Right         ; cText := "Para a direita"                        ; EXIT
        CASE LS_View          ; cText := "Exibir"                                ; EXIT
        CASE LS_MenuBar       ; cText := "Barra de menú"                         ; EXIT
        CASE LS_Toolbar       ; cText := "Barra de ferramentas"                  ; EXIT
        CASE LS_Bookmarks     ; cText := "Marcadores"                            ; EXIT
        CASE LS_Show          ; cText := "Exibir"                                ; EXIT
        CASE LS_ExpandAll     ; cText := "Expandir todos"                        ; EXIT
        CASE LS_CollapseAll   ; cText := "Recolher todos"                        ; EXIT
        CASE LS_Settings      ; cText := "Configurações"                         ; EXIT
        CASE LS_OpenAtOnce    ; cText := "Abrir ao click"                        ; EXIT
          CASE LS_RestSession   ; cText := "&Restore last session on start"        ; EXIT
          CASE LS_TabGoToFile   ; cText := "&Changing tab, go to file"             ; EXIT
        CASE LS_EscExit       ; cText := "Esc - Fechar o programa"               ; EXIT
        CASE LS_SumatraDir    ; cText := "Pasta do SumatraPDF"                   ; EXIT
        CASE LS_PDFtkDir      ; cText := "Pasta do PDFtk"                        ; EXIT
        CASE LS_Language      ; cText := "Idiomas"                               ; EXIT
          CASE LS_AboutPdfView  ; cText := "About PdfView"                         ; EXIT
          CASE LS_AboutSumatra  ; cText := "About SumatraPDF"                      ; EXIT
        CASE LS_AppRunning    ; cText := "O programa já está em execução!"       ; EXIT
          CASE LS_Disk          ; cText := "Disk"                                  ; EXIT
        CASE LS_NoDisk        ; cText := "O disco está inacessível!"             ; EXIT
        CASE LS_NoFile        ; cText := "O arquivo não existe!"                 ; EXIT
        CASE LS_SetPathTo     ; cText := "Defina o caminho para"                 ; EXIT
        CASE LS_ListRefresh   ; cText := "A lista será atualizada."              ; EXIT
        CASE LS_OpenFilePage  ; cText := "Abrir arquivo na página"               ; EXIT
        CASE LS_PageNum       ; cText := "Número de página"                      ; EXIT
        CASE LS_SelSumatraDir ; cText := "Selecione a pasta do SumatraPDF"       ; EXIT
        CASE LS_SelPDFtkDir   ; cText := "Selecione a pasta do PDFtk"            ; EXIT
          CASE LS_OnlyNames     ; cText := "Only &names"                           ; EXIT
          CASE LS_FilesAmount   ; cText := "Amount of &files"                      ; EXIT
          CASE LS_Selected      ; cText := "&Selected"                             ; EXIT
          CASE LS_NonExistent   ; cText := "&Non-existent"                         ; EXIT
          CASE LS_All           ; cText := "&All"                                  ; EXIT
          CASE LS_OutputDir     ; cText := "Output directory"                      ; EXIT
          CASE LS_TargetFiles   ; cText := "Target files"                          ; EXIT
          CASE LS_CantRunPDFtk  ; cText := "Can not run PDFtk.exe!"                ; EXIT
          CASE LS_PDFtkError    ; cText := "PDFtk error"                           ; EXIT
          CASE LS_OwnerPassReq  ; cText := "Owner password required"               ; EXIT
          CASE LS_ShowPass      ; cText := "Show password"                         ; EXIT
          CASE LS_Wait          ; cText := "Wait"                                  ; EXIT
          CASE LS_PDFtkWorking  ; cText := "PDFtk is working"                      ; EXIT
          CASE LS_CopySource    ; cText := "copying source files"                  ; EXIT
          CASE LS_CopyTarget    ; cText := "copying target files"                  ; EXIT
          CASE LS_PdfViewUsing  ; cText := "PDF viewer, merger and splitter using" ; EXIT
          CASE LS_DevelopedIn   ; cText := "Developed in"                          ; EXIT
          CASE LS_Author        ; cText := "Author"                                ; EXIT
          CASE LS_Translators   ; cText := "Translators"                           ; EXIT
        CASE LS_Open          ; cText := "Abrir"                                 ; EXIT
          CASE LS_Remove        ; cText := "&Remove"                               ; EXIT
        CASE LS_Browse        ; cText := "Localizar"                             ; EXIT
        CASE LS_OK            ; cText := "Ok"                                    ; EXIT
        CASE LS_Cancel        ; cText := "Cancelar"                              ; EXIT
      ENDSWITCH
      EXIT

    CASE "ru"
      SWITCH nStr
        CASE LS_File          ; cText := "файл"                                      ; EXIT
        CASE LS_OpenCurTab    ; cText := "Открыть в текущей вкладке"                 ; EXIT
        CASE LS_OpenNewTab    ; cText := "Открыть в новой вкладке"                   ; EXIT
        CASE LS_OpenPage      ; cText := "Открыть на странице"                       ; EXIT
        CASE LS_OpenPageTab   ; cText := "Открыть на странице в новой вкладке"       ; EXIT
        CASE LS_OpenSession   ; cText := "Открыть последнюю сессию"                  ; EXIT
        CASE LS_RecentFiles   ; cText := "Недавние файлы"                            ; EXIT
        CASE LS_GoToSubDir    ; cText := "Перейти в подкаталог"                      ; EXIT
        CASE LS_GoToParentDir ; cText := "Перейти в родительский каталог"            ; EXIT
        CASE LS_ChooseDir     ; cText := "Выберите каталог"                          ; EXIT
        CASE LS_RefreshList   ; cText := "Обновить список"                           ; EXIT
        CASE LS_Exit          ; cText := "Выход"                                     ; EXIT
        CASE LS_Document      ; cText := "Документ"                                  ; EXIT
        CASE LS_SaveAs        ; cText := "Сохранить как"                             ; EXIT
        CASE LS_Print         ; cText := "Печать"                                    ; EXIT
        CASE LS_Proper        ; cText := "Свойства"                                  ; EXIT
        CASE LS_Close         ; cText := "Закрыть"                                   ; EXIT
        CASE LS_SplitPages    ; cText := "Разделить на отдельные страницы"           ; EXIT
        CASE LS_ChooseDoc     ; cText := "Выбрать документ"                          ; EXIT
        CASE LS_GoToFile      ; cText := "Перейти к файлу"                           ; EXIT
        CASE LS_Page          ; cText := "Страница"                                  ; EXIT
        CASE LS_GoTo          ; cText := "Перейти к"                                 ; EXIT
        CASE LS_Prev          ; cText := "Предыдущая"                                ; EXIT
        CASE LS_Next          ; cText := "Следущая"                                  ; EXIT
        CASE LS_First         ; cText := "Первая"                                    ; EXIT
        CASE LS_Last          ; cText := "Последняя"                                 ; EXIT
        CASE LS_Find          ; cText := "Найти"                                     ; EXIT
        CASE LS_Text          ; cText := "Текст"                                     ; EXIT
        CASE LS_PrevOccur     ; cText := "Предыдущее вхождение"                      ; EXIT
        CASE LS_NextOccur     ; cText := "Следующее вхождение"                       ; EXIT
        CASE LS_Zoom          ; cText := "Масштаб"                                   ; EXIT
        CASE LS_SizeDn        ; cText := "Увеличить"                                 ; EXIT
        CASE LS_SizeUp        ; cText := "Уменьшить"                                 ; EXIT
        CASE LS_ZoomFactor    ; cText := "Указать масштаб"                           ; EXIT
        CASE LS_FitPage       ; cText := "По размеру страницы"                       ; EXIT
        CASE LS_ActualSize    ; cText := "Настоящий размер"                          ; EXIT
        CASE LS_FitWidth      ; cText := "По ширине"                                 ; EXIT
        CASE LS_Rotate        ; cText := "Вращать"                                   ; EXIT
        CASE LS_Left          ; cText := "В лево"                                    ; EXIT
        CASE LS_Right         ; cText := "В право"                                   ; EXIT
        CASE LS_View          ; cText := "Вид"                                       ; EXIT
        CASE LS_MenuBar       ; cText := "Панель меню"                               ; EXIT
        CASE LS_Toolbar       ; cText := "Панель инструментов"                       ; EXIT
        CASE LS_Bookmarks     ; cText := "Закладки PDF"                              ; EXIT
        CASE LS_Show          ; cText := "Показать"                                  ; EXIT
        CASE LS_ExpandAll     ; cText := "Расширить все"                             ; EXIT
        CASE LS_CollapseAll   ; cText := "Свернуть все"                              ; EXIT
        CASE LS_Settings      ; cText := "Настройки"                                 ; EXIT
        CASE LS_OpenAtOnce    ; cText := "Открыть PDF немедленно"                    ; EXIT
        CASE LS_RestSession   ; cText := "Восстановить последнюю сессию на старте"   ; EXIT
        CASE LS_TabGoToFile   ; cText := "Изменение вкладки, перейдите к файлу"      ; EXIT
        CASE LS_EscExit       ; cText := "Esc - выход"                               ; EXIT
        CASE LS_SumatraDir    ; cText := "Каталог SumatraPDF"                        ; EXIT
        CASE LS_PDFtkDir      ; cText := "Каталог PDFtk"                             ; EXIT
        CASE LS_Language      ; cText := "Язык"                                      ; EXIT
        CASE LS_AboutPdfView  ; cText := "О программе PdfView"                       ; EXIT
        CASE LS_AboutSumatra  ; cText := "О программе SumatraPDF"                    ; EXIT
        CASE LS_AppRunning    ; cText := "Программа уже работает!"                   ; EXIT
        CASE LS_Disk          ; cText := "Диск"                                      ; EXIT
        CASE LS_NoDisk        ; cText := "Диск не доступен!"                         ; EXIT
        CASE LS_NoFile        ; cText := "Файл не существует!"                       ; EXIT
        CASE LS_SetPathTo     ; cText := "Установить путь к"                         ; EXIT
        CASE LS_ListRefresh   ; cText := "Список будет обновлен."                    ; EXIT
        CASE LS_OpenFilePage  ; cText := "Открыть файл на странице"                  ; EXIT
        CASE LS_PageNum       ; cText := "Номер страницы"                            ; EXIT
        CASE LS_SelSumatraDir ; cText := "Выберите папку SumatraPDF"                 ; EXIT
        CASE LS_SelPDFtkDir   ; cText := "Выберите папку PDFtk"                      ; EXIT
        CASE LS_OnlyNames     ; cText := "Только имена"                              ; EXIT
        CASE LS_FilesAmount   ; cText := "Количество файлов"                         ; EXIT
        CASE LS_Selected      ; cText := "Выбранный"                                 ; EXIT
        CASE LS_NonExistent   ; cText := "Несуществующий"                            ; EXIT
        CASE LS_All           ; cText := "Все"                                       ; EXIT
        CASE LS_OutputDir     ; cText := "Каталог назначения"                        ; EXIT
        CASE LS_TargetFiles   ; cText := "Файлы целевые"                             ; EXIT
        CASE LS_CantRunPDFtk  ; cText := "Не могу запустить PDFtk.exe!"              ; EXIT
        CASE LS_PDFtkError    ; cText := "Ошибка Pdftk"                              ; EXIT
        CASE LS_OwnerPassReq  ; cText := "Требуется пароль владельца"                ; EXIT
        CASE LS_ShowPass      ; cText := "Показать пароль"                           ; EXIT
        CASE LS_Wait          ; cText := "Подожди"                                   ; EXIT
        CASE LS_PDFtkWorking  ; cText := "PDFtk работает"                            ; EXIT
        CASE LS_CopySource    ; cText := "копирование исходных файлов"               ; EXIT
        CASE LS_CopyTarget    ; cText := "копирование целевых файлов"                ; EXIT
        CASE LS_PdfViewUsing  ; cText := "Просмотр, слияние и деление PDF с помощью" ; EXIT
        CASE LS_DevelopedIn   ; cText := "Разработано в"                             ; EXIT
        CASE LS_Author        ; cText := "Автор"                                     ; EXIT
        CASE LS_Translators   ; cText := "Переводчики"                               ; EXIT
        CASE LS_Open          ; cText := "Открыть"                                   ; EXIT
        CASE LS_Remove        ; cText := "Удалить"                                   ; EXIT
        CASE LS_Browse        ; cText := "Выбрать"                                   ; EXIT
        CASE LS_OK            ; cText := "OK"                                        ; EXIT
        CASE LS_Cancel        ; cText := "Отмена"                                    ; EXIT
      ENDSWITCH
      EXIT

    //Spanish - translated by Pablo César
    CASE "es"
      SWITCH nStr
        CASE LS_File          ; cText := "Archivo"                               ; EXIT
          CASE LS_OpenCurTab    ; cText := "&Open in current tab"                  ; EXIT
          CASE LS_OpenNewTab    ; cText := "Open in &new tab"                      ; EXIT
        CASE LS_OpenPage      ; cText := "Abrir archivo en la página"            ; EXIT
          CASE LS_OpenPageTab   ; cText := "Open at page in new &tab"              ; EXIT
          CASE LS_OpenSession   ; cText := "Open last &session"                    ; EXIT
          CASE LS_RecentFiles   ; cText := "Recent &files"                         ; EXIT
          CASE LS_GoToSubDir    ; cText := "Go to subdirectory"                    ; EXIT
          CASE LS_GoToParentDir ; cText := "Go to parent directory"                ; EXIT
          CASE LS_ChooseDir     ; cText := "Choose &directory"                     ; EXIT
        CASE LS_RefreshList   ; cText := "Actualizar"                            ; EXIT
        CASE LS_Exit          ; cText := "Salir"                                 ; EXIT
          CASE LS_Document      ; cText := "&Document"                             ; EXIT
        CASE LS_SaveAs        ; cText := "Guardar como"                          ; EXIT
        CASE LS_Print         ; cText := "Impresión"                             ; EXIT
        CASE LS_Proper        ; cText := "Propriedades"                          ; EXIT
        CASE LS_Close         ; cText := "Cerrar"                                ; EXIT
          CASE LS_SplitPages  ; cText := "Split into single pages"               ; EXIT
          CASE LS_ChooseDoc     ; cText := "Choose &document"                      ; EXIT
          CASE LS_GoToFile      ; cText := "Go to &file"                           ; EXIT
        CASE LS_Page          ; cText := "Página"                                ; EXIT
        CASE LS_GoTo          ; cText := "Ir para"                               ; EXIT
        CASE LS_Prev          ; cText := "Anterior"                              ; EXIT
        CASE LS_Next          ; cText := "Próximo"                               ; EXIT
        CASE LS_First         ; cText := "Primer"                                ; EXIT
        CASE LS_Last          ; cText := "Ultimo"                                ; EXIT
        CASE LS_Find          ; cText := "Procurar"                              ; EXIT
        CASE LS_Text          ; cText := "Texto"                                 ; EXIT
        CASE LS_PrevOccur     ; cText := "Ocurrencia previa"                     ; EXIT
        CASE LS_NextOccur     ; cText := "Ocurrencia posterior"                  ; EXIT
        CASE LS_Zoom          ; cText := "Visualización"                         ; EXIT
        CASE LS_SizeDn        ; cText := "Disminuir"                             ; EXIT
        CASE LS_SizeUp        ; cText := "Aumentar"                              ; EXIT
        CASE LS_ZoomFactor    ; cText := "Nivel del zoom"                        ; EXIT
        CASE LS_FitPage       ; cText := "Ajuste de página"                      ; EXIT
        CASE LS_ActualSize    ; cText := "Zoom automático"                       ; EXIT
        CASE LS_FitWidth      ; cText := "Ajuste del ancho"                      ; EXIT
        CASE LS_Rotate        ; cText := "Girar"                                 ; EXIT
        CASE LS_Left          ; cText := "Hacia la izquierda"                    ; EXIT
        CASE LS_Right         ; cText := "Hacia la derecha"                      ; EXIT
        CASE LS_View          ; cText := "Exibición"                             ; EXIT
        CASE LS_MenuBar       ; cText := "Barra de menú"                         ; EXIT
        CASE LS_Toolbar       ; cText := "Barra de herramientas"                 ; EXIT
        CASE LS_Bookmarks     ; cText := "Marcadores"                            ; EXIT
        CASE LS_Show          ; cText := "Exibir"                                ; EXIT
        CASE LS_ExpandAll     ; cText := "Expandir todos"                        ; EXIT
        CASE LS_CollapseAll   ; cText := "Retraer todos"                         ; EXIT
        CASE LS_Settings      ; cText := "Configuraciones"                       ; EXIT
        CASE LS_OpenAtOnce    ; cText := "Abrir de inmediato"                    ; EXIT
          CASE LS_RestSession   ; cText := "&Restore last session on start"        ; EXIT
          CASE LS_TabGoToFile   ; cText := "&Changing tab, go to file"             ; EXIT
        CASE LS_EscExit       ; cText := "Esc - Cierre del programa"             ; EXIT
        CASE LS_SumatraDir    ; cText := "Carpeta del SumatraPDF"                ; EXIT
        CASE LS_PDFtkDir      ; cText := "Carpeta del PDFtkDir"                  ; EXIT
        CASE LS_Language      ; cText := "Idiomas"                               ; EXIT
          CASE LS_AboutPdfView  ; cText := "About PdfView"                         ; EXIT
          CASE LS_AboutSumatra  ; cText := "About SumatraPDF"                      ; EXIT
        CASE LS_AppRunning    ; cText := "El programa ya está en marcha!"        ; EXIT
          CASE LS_Disk          ; cText := "Disk"                                  ; EXIT
        CASE LS_NoDisk        ; cText := "El disco es inaccesible!"              ; EXIT
        CASE LS_NoFile        ; cText := "El archivo no existe!"                 ; EXIT
        CASE LS_SetPathTo     ; cText := "Fije la ruta"                          ; EXIT
        CASE LS_ListRefresh   ; cText := "Se actualizará la lista."              ; EXIT
        CASE LS_OpenFilePage  ; cText := "Abrir archivo en la página"            ; EXIT
        CASE LS_PageNum       ; cText := "Número de página"                      ; EXIT
        CASE LS_SelSumatraDir ; cText := "Seleccione la carpeta SumatraPDF"      ; EXIT
        CASE LS_SelPDFtkDir   ; cText := "Seleccione la carpeta PDFtk"           ; EXIT
          CASE LS_OnlyNames     ; cText := "Only &names"                           ; EXIT
          CASE LS_FilesAmount   ; cText := "Amount of &files"                      ; EXIT
          CASE LS_Selected      ; cText := "&Selected"                             ; EXIT
          CASE LS_NonExistent   ; cText := "&Non-existent"                         ; EXIT
          CASE LS_All           ; cText := "&All"                                  ; EXIT
          CASE LS_OutputDir     ; cText := "Output directory"                      ; EXIT
          CASE LS_TargetFiles   ; cText := "Target files"                          ; EXIT
          CASE LS_CantRunPDFtk  ; cText := "Can not run PDFtk.exe!"                ; EXIT
          CASE LS_PDFtkError    ; cText := "PDFtk error"                           ; EXIT
          CASE LS_OwnerPassReq  ; cText := "Owner password required"               ; EXIT
          CASE LS_ShowPass      ; cText := "Show password"                         ; EXIT
          CASE LS_Wait          ; cText := "Wait"                                  ; EXIT
          CASE LS_PDFtkWorking  ; cText := "PDFtk is working"                      ; EXIT
          CASE LS_CopySource    ; cText := "copying source files"                  ; EXIT
          CASE LS_CopyTarget    ; cText := "copying target files"                  ; EXIT
          CASE LS_PdfViewUsing  ; cText := "PDF viewer, merger and splitter using" ; EXIT
          CASE LS_DevelopedIn   ; cText := "Developed in"                          ; EXIT
          CASE LS_Author        ; cText := "Author"                                ; EXIT
          CASE LS_Translators   ; cText := "Translators"                           ; EXIT
        CASE LS_Open          ; cText := "Abrir"                                 ; EXIT
          CASE LS_Remove        ; cText := "&Remove"                               ; EXIT
        CASE LS_Browse        ; cText := "Buscar"                                ; EXIT
        CASE LS_OK            ; cText := "Ok"                                    ; EXIT
        CASE LS_Cancel        ; cText := "Cancelar"                              ; EXIT
      ENDSWITCH
      EXIT
  ENDSWITCH

  IF Empty(cText)
    SWITCH nStr
      CASE LS_File          ; cText := "&File"                                 ; EXIT
      CASE LS_OpenCurTab    ; cText := "&Open in current tab"                  ; EXIT
      CASE LS_OpenNewTab    ; cText := "Open in &new tab"                      ; EXIT
      CASE LS_OpenPage      ; cText := "Open at &page"                         ; EXIT
      CASE LS_OpenPageTab   ; cText := "Open at page in new &tab"              ; EXIT
      CASE LS_OpenSession   ; cText := "Open last &session"                    ; EXIT
      CASE LS_RecentFiles   ; cText := "Recent &files"                         ; EXIT
      CASE LS_GoToSubDir    ; cText := "Go to subdirectory"                    ; EXIT
      CASE LS_GoToParentDir ; cText := "Go to parent directory"                ; EXIT
      CASE LS_ChooseDir     ; cText := "Choose &directory"                     ; EXIT
      CASE LS_RefreshList   ; cText := "&Refresh list"                         ; EXIT
      CASE LS_Exit          ; cText := "E&xit"                                 ; EXIT
      CASE LS_Document      ; cText := "&Document"                             ; EXIT
      CASE LS_SaveAs        ; cText := "&Save as"                              ; EXIT
      CASE LS_Print         ; cText := "&Print"                                ; EXIT
      CASE LS_Proper        ; cText := "P&roperties"                           ; EXIT
      CASE LS_Close         ; cText := "&Close"                                ; EXIT
      CASE LS_SplitPages    ; cText := "Split into single pages"               ; EXIT
      CASE LS_ChooseDoc     ; cText := "Choose &document"                      ; EXIT
      CASE LS_GoToFile      ; cText := "Go to &file"                           ; EXIT
      CASE LS_Page          ; cText := "&Page"                                 ; EXIT
      CASE LS_GoTo          ; cText := "&Go to"                                ; EXIT
      CASE LS_Prev          ; cText := "&Previous"                             ; EXIT
      CASE LS_Next          ; cText := "&Next"                                 ; EXIT
      CASE LS_First         ; cText := "&First"                                ; EXIT
      CASE LS_Last          ; cText := "&Last"                                 ; EXIT
      CASE LS_Find          ; cText := "Fi&nd"                                 ; EXIT
      CASE LS_Text          ; cText := "&Text"                                 ; EXIT
      CASE LS_PrevOccur     ; cText := "&Previous occurence"                   ; EXIT
      CASE LS_NextOccur     ; cText := "&Next occurence"                       ; EXIT
      CASE LS_Zoom          ; cText := "&Zoom"                                 ; EXIT
      CASE LS_SizeDn        ; cText := "Size &down"                            ; EXIT
      CASE LS_SizeUp        ; cText := "Size &up"                              ; EXIT
      CASE LS_ZoomFactor    ; cText := "&Zoom factor"                          ; EXIT
      CASE LS_FitPage       ; cText := "Fit &page"                             ; EXIT
      CASE LS_ActualSize    ; cText := "&Actual size"                          ; EXIT
      CASE LS_FitWidth      ; cText := "Fit &width"                            ; EXIT
      CASE LS_Rotate        ; cText := "&Rotate"                               ; EXIT
      CASE LS_Left          ; cText := "&Left"                                 ; EXIT
      CASE LS_Right         ; cText := "&Right"                                ; EXIT
      CASE LS_View          ; cText := "&View"                                 ; EXIT
      CASE LS_MenuBar       ; cText := "&Menu bar"                             ; EXIT
      CASE LS_Toolbar       ; cText := "&Toolbar"                              ; EXIT
      CASE LS_Bookmarks     ; cText := "PDF &bookmarks"                        ; EXIT
      CASE LS_Show          ; cText := "&Show"                                 ; EXIT
      CASE LS_ExpandAll     ; cText := "&Expand all"                           ; EXIT
      CASE LS_CollapseAll   ; cText := "&Collapse all"                         ; EXIT
      CASE LS_Settings      ; cText := "&Settings"                             ; EXIT
      CASE LS_OpenAtOnce    ; cText := "&Open PDF immediately"                 ; EXIT
      CASE LS_RestSession   ; cText := "&Restore last session on start"        ; EXIT
      CASE LS_TabGoToFile   ; cText := "&Changing tab, go to file"             ; EXIT
      CASE LS_EscExit       ; cText := "&Esc - exit program"                   ; EXIT
      CASE LS_SumatraDir    ; cText := "&SumatraPDF directory"                 ; EXIT
      CASE LS_PDFtkDir      ; cText := "&PDFtk directory"                      ; EXIT
      CASE LS_Language      ; cText := "&Language"                             ; EXIT
      CASE LS_AboutPdfView  ; cText := "About PdfView"                         ; EXIT
      CASE LS_AboutSumatra  ; cText := "About SumatraPDF"                      ; EXIT
      CASE LS_AppRunning    ; cText := "Program already is running!"           ; EXIT
      CASE LS_Disk          ; cText := "Disk"                                  ; EXIT
      CASE LS_NoDisk        ; cText := "Disk is not available!"                ; EXIT
      CASE LS_NoFile        ; cText := "File does not exist!"                  ; EXIT
      CASE LS_SetPathTo     ; cText := "Set path to"                           ; EXIT
      CASE LS_ListRefresh   ; cText := "List will be refreshed."               ; EXIT
      CASE LS_OpenFilePage  ; cText := "Open file at page"                     ; EXIT
      CASE LS_PageNum       ; cText := "Page number"                           ; EXIT
      CASE LS_SelSumatraDir ; cText := "Select SumatraPDF directory"           ; EXIT
      CASE LS_SelPDFtkDir   ; cText := "Select PDFtk directory"                ; EXIT
      CASE LS_OnlyNames     ; cText := "Only &names"                           ; EXIT
      CASE LS_FilesAmount   ; cText := "Amount of &files"                      ; EXIT
      CASE LS_Selected      ; cText := "&Selected"                             ; EXIT
      CASE LS_NonExistent   ; cText := "&Non-existent"                         ; EXIT
      CASE LS_All           ; cText := "&All"                                  ; EXIT
      CASE LS_OutputDir     ; cText := "Output directory"                      ; EXIT
      CASE LS_TargetFiles   ; cText := "Target files"                          ; EXIT
      CASE LS_CantRunPDFtk  ; cText := "Can not run PDFtk.exe!"                ; EXIT
      CASE LS_PDFtkError    ; cText := "PDFtk error"                           ; EXIT
      CASE LS_OwnerPassReq  ; cText := "Owner password required"               ; EXIT
      CASE LS_ShowPass      ; cText := "Show password"                         ; EXIT
      CASE LS_Wait          ; cText := "Wait"                                  ; EXIT
      CASE LS_PDFtkWorking  ; cText := "PDFtk is working"                      ; EXIT
      CASE LS_CopySource    ; cText := "copying source files"                  ; EXIT
      CASE LS_CopyTarget    ; cText := "copying target files"                  ; EXIT
      CASE LS_PdfViewUsing  ; cText := "PDF viewer, merger and splitter using" ; EXIT
      CASE LS_DevelopedIn   ; cText := "Developed in"                          ; EXIT
      CASE LS_Author        ; cText := "Author"                                ; EXIT
      CASE LS_Translators   ; cText := "Translators"                           ; EXIT
      CASE LS_Open          ; cText := "&Open"                                 ; EXIT
      CASE LS_Remove        ; cText := "&Remove"                               ; EXIT
      CASE LS_Browse        ; cText := "&Browse"                               ; EXIT
      CASE LS_OK            ; cText := "OK"                                    ; EXIT
      CASE LS_Cancel        ; cText := "&Cancel"                               ; EXIT
    ENDSWITCH
  ENDIF

  IF lRemoveAmpersand == .T.
    cText := StrTran(cText, "&", "", 1, 1)
  ENDIF

RETURN cText
